/*******************************************************************************
*	FUNCTION: 	Cdisplay.c
*	LIBRARY:  	display
*	AUTHOR:	  	Scott Streit
*	DATE:     	11-03-1999
*	VERSION:  	1
*
*	DESCRIPTION:	This native method displays an image in an open
*			window, such as SAOImage and Ximtool.
*
*	PARAMETERS:	mem_fd   File descriptor that references the image.
*			rows	 The number of rows in the image.
*			cols	 The number of columns in the image.
*
*	RETURNS:	Returns the reply from the display package. Throws
*			an Exception if a connection to the display server
*			(window) could not be established.
*******************************************************************************/
#include <stdlib.h>
#include "ImageDisplay.h"
#include "which_system.h"

JNIEXPORT jint JNICALL
Java_ImageDisplay_Cdisplay(JNIEnv *env, jclass clazz, jint mem_fd, jint rows, jint cols)
{
	CDLPtr cdl;
	jclass newExcCls;
	int reply = 0;
	unsigned char *pix = NULL;
		
	/*
	   Create the temporary image display buffer. NOTE: THIS BUFFER MUST BE
	   USED BECAUSE SAOIMAGE DESTROYS THE DATA IN THE BUFFER ... WHO KNOWS
   	   FOR WHAT REASON, BUT IT DOES. WITHOUT THIS, THE SAVED IMAGE WILL
   	   BE WRONG.
   	*/
	pix = (unsigned char *)malloc((int)rows*(int)cols*2);
	memcpy(pix, (unsigned char *)mem_fd, rows*cols*2);

	/* Open the package and a connection to the server. */
        if (!(cdl = cdl_open ((char *)getenv("IMTDEV"))) ) {
			newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        		/* Unable to find the new exception class, give up. */
        		if (newExcCls == 0)
        			return;

       			(*env)->ThrowNew(env, newExcCls, "ERROR: Could not open connection to dislay server \n");
       			return 1;
        }

	else {
		reply = cdl_displayPix(cdl, (unsigned char *)pix, cols, rows, 16, 1, FB_AUTO, 1);
				
		/* Close the package and the connection to the server. */
		cdl_close(cdl);

		free(pix);
	}

	return reply;
}
