/*******************************************************************************
*	FUNCTION: 	Cclose.c
*	LIBRARY:  	driver
*	AUTHOR:	  	Scott Streit
*	DATE:     	11-03-1999
*	VERSION:  	1
*
*	DESCRIPTION:	This native method is used to close a connection
*			to the PCI device driver.
*
*	PARAMETERS:	pci_fd	   The reference to the pci board that was
*				   obtained by invoking Copen().
*
*	RETURNS:	A zero is returned if the close is successful. For
*			a complete listing of errors, see the man pages for
*			"close". A java/lang/Exception is thrown if an error
*			is found.
*******************************************************************************/
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <jni.h>
#include "VoodooSetup.h"

JNIEXPORT jint JNICALL
Java_VoodooSetup_Cclose(JNIEnv *env, jclass clazz, jint pci_fd)
{
	int value;
 	jclass newExcCls;
	char message[100];
	
	value = close(pci_fd);
	
	/* Check for an error. */
	if (value == -1) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		sprintf(message, "Error While Closing Device Driver Connection (errno = %d) \n%s", errno, strerror(errno));
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }

 	return value;
}
