/*******************************************************************************
*	FUNCTION: 	Cioctl.c
*	LIBRARY:  	driver
*	AUTHOR:	  	Scott Streit
*	DATE:     	07-25-2000
*	VERSION:  	2
*
*	DESCRIPTION:	This native method is used to send control commands
*			to the camera and other special functions.
*
*	PARAMETERS:	pci_fd	     File descriptor returned from Copen.
*			data	     Any input parameter.
*			pci_request  A valid ioctl command for the PCI board.
*
*	RETURNS:	The approprate value is returned (if there is a return
*			value) to the user. An Exception is thrown if a problem
*			occurs while transfering data between the user and the
*  			device driver. In this case, a "-1" and EFAULT are
*			returned from ioctl. An Exception also occurs if the
*			PCI DSP FIFO is not ready to accept commands and a set
*			number of attempts fail. In this case, a "-1" and EIO
*			are returned from ioctl.
*******************************************************************************/
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <jni.h>
#include "VoodooSetup.h"

JNIEXPORT jint JNICALL
Java_VoodooSetup_Cioctl(JNIEnv *env, jclass jclazz, jint pci_fd, jint pci_request, jint data)
{
	int value;
	jclass newExcCls;
	char message[100];

	value = ioctl(pci_fd, pci_request, &data);
  
	/* Check for an error. */
	if (value == -1 && errno == EIO) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		(*env)->ThrowNew(env, newExcCls, "PCI Board Not Ready To Process Commands.");
        }

	else if (value == -1 && errno == EFAULT) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		sprintf(message, "Error While Transfering Data (errno = %d) \n%s", errno, strerror(errno));
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }

	return data;
}
