/*******************************************************************************
*	FUNCTION: 	Copen.c
*	LIBRARY:  	driver
*	AUTHOR:	  	Scott Streit
*	DATE:     	11-03-1999
*	VERSION:  	1
*
*	DESCRIPTION:	This native method is used to open a connection
*			to the PCI device driver. This method must be
*			successful before any access to the PCI board is
*			allowed.
*
*	PARAMETERS:	pci_dev	   The name of the device to open.
*
*	RETURNS:	The file descriptor is returned if the open is
*			successful. For a complete listing of errors, see
*			the man pages for "open". A java/lang/Exception is
*			thrown if either the Reply or Image Buffers cannot
*			be allocated.
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <jni.h>
#include "VoodooSetup.h"

JNIEXPORT jint JNICALL
Java_VoodooSetup_Copen(JNIEnv *env, jobject obj, jstring pci_dev)
{
	int pci_fd;
	jclass newExcCls;
	char message[100];
	
	/* Get the name of the device to open. */
	const char *str = (*env)->GetStringUTFChars(env, pci_dev, 0);
 
	/* Try to open a connection. */
	pci_fd = open(str, O_RDWR);
  
        /* Check for Reply or Image Buffer allocation failure. */
	if (pci_fd == -1 && errno == ENOMEM) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;
  
        	sprintf(message, "Required Buffers (Reply or Image) \n Could Not Be Allocated (errno = %d) \n%s", errno, strerror(errno));

        	(*env)->ThrowNew(env, newExcCls, message);
        }

	/* Check that the device can be opened. */
	else if (pci_fd == -1) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;
  
        	sprintf(message, "Device Could Not Be Opened (errno = %d) \n%s", errno, strerror(errno));
        	
        	(*env)->ThrowNew(env, newExcCls, message);
        }
       
        /* Free the allocated memory for the device name. */    
	(*env)->ReleaseStringUTFChars(env, pci_dev, str);

	return pci_fd;
}
