/*******************************************************************************
*	FUNCTION: 	Cread_image.c
*	LIBRARY:  	driver
*	AUTHOR:	  	Scott Streit
*	DATE:     	11-03-1999
*	VERSION:  	1
*
*	DESCRIPTION:	This native method is used to read image data
*			from the camera.
*
*	PARAMETERS:	pci_fd	   File descriptor returned from Copen.
*			fd	   A reference to the user's image buffers.
*			read_size  The image size in bytes.
*
*	RETURNS:	The image data is returned to the specified buffer.
*			A java/lang/Exception is thrown if the DMA times out
*			or an image buffer ends up in a unexpected state.
*******************************************************************************/
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <jni.h>
#include "VoodooSetup.h"

JNIEXPORT jint JNICALL
Java_VoodooSetup_Cread_1image(JNIEnv *env, jclass clazz, jint pci_fd, jint fd, jint read_size)

{
	int value = 0;
	jclass newExcCls;
	char message[100];
	
  	/* Obtain a pointer to the user's image buffer. */
  	void *addrPtr = (void *)fd;
  
  	/* Call the device driver "read" entry point. */
  	value = read(pci_fd, addrPtr, read_size);

	/* Check for a DMA timeout. */
	if (value == -1 && errno == EIO) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

		printf("\n Throwing exception");
		
       		sprintf(message, "DMA Timed Out While Waiting For Image Data (errno = %d) \n%s", errno, strerror(errno));
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }

	/* Check for abort readout. */
	else if (value == -1 && errno == EINTR) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		sprintf(message, "Image Readout Successfully Aborted");
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }

	/* Check for image buffer state corruption. */
	else if (value == -1 && errno == ECHILD) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		sprintf(message, "Image Buffer Is In A Corrupt State (errno = %d) \n%s", errno, strerror(errno));
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }

	/* Check for some other error. */
	else if (value == -1) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		if (errno == 0)
       			return;
       			
       		sprintf(message, "Error While Reading Image (errno = %d) \n%s", errno, strerror(errno));
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }

	return value;
}
