/*******************************************************************************
*	FUNCTION: 	Cread_reply.c
*	LIBRARY:  	driver
*	AUTHOR:	  	Scott Streit
*	DATE:     	07-25-2000
*	VERSION:  	2
*
*	DESCRIPTION:	This native method is used to read a reply
*			from the camera.
*
*	PARAMETERS:	pci_fd	   File descriptor returned from Copen.
*			reply_arg  Any input parameter.
*
*	RETURNS:	The reply value is returned to the user. An
*			Exception is thrown if a problem occurs while
*			transfering data between the user and the device
*			driver. 
*******************************************************************************/
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <jni.h>
#include "VoodooSetup.h"

#define ASTROPCI_GET_REPLY	0x3

JNIEXPORT jint JNICALL
Java_VoodooSetup_Cread_1reply(JNIEnv *env, jclass clazz, jint pci_fd, jint reply_arg)
{
	int value;
	jclass newExcCls;
	char message[100];

	/* Call the device driver "ioctl" entry point. */
	value = ioctl(pci_fd, ASTROPCI_GET_REPLY, &reply_arg);

	/* Check for an error. */
	if (value == -1 && errno == EFAULT) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		sprintf(message, "Error While Transfering Reply Data (errno = %d) \n%s", errno, strerror(errno));
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }

	return reply_arg;
}
