/*******************************************************************************
*	FUNCTION: 	Ccreate_memory.c
*	LIBRARY:  	mem
*	AUTHOR:	  	Scott Streit
*	DATE:     	11-03-1999
*	VERSION:  	1
*
*	DESCRIPTION:	This native method is used to create an image buffer
*			for the main application.
*
*	PARAMETERS:	buffer_size  The size of the buffer (in bytes) to be
*				     allocated.
*
*	RETURNS:	A reference to the image buffer is returned. An
*			Exception is thrown if the image buffers cannot be
*			allocated.
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include "Cmemory.h"
#include "MemorySetup.h"

JNIEXPORT jint JNICALL Java_MemorySetup_Ccreate_1memory(JNIEnv *env, jclass clazz, jint buffer_size)
{
	unsigned char *fd;
	int x;
	jclass newExcCls;
	char message[100];

	printf("\nBuffers Allocated ...\n");
	
	if (addrPtr != NULL) {
		/* Free the image buffer memory */
		free(fd);
	}
			
	/* Create the image buffer. */
	fd = (unsigned char *)malloc(buffer_size);

	/* Check for an out of physical memory error. */
	if (fd == NULL && errno == ENOMEM) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		(*env)->ThrowNew(env, newExcCls, "Error Allocating Image Buffers. The Systems \n Physical Limits Have Been Exceeded. \n");
        }

	/* Check for an out of memory error. */
	else if (fd == NULL && errno == EAGAIN) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		(*env)->ThrowNew(env, newExcCls, "Error Allocating Image Buffers. \n Not Enough Memory CURRENTLY Available. \n");
        }
	
	/* Check for any other errors. */
	else if (fd == NULL) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		sprintf(message, "Error Allocating Memory (errno = %d) \n", errno);
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }

	return (int)fd;
}

