/*******************************************************************************
*	FUNCTION: 	Cdata_check.c
*	LIBRARY:  	mem
*	AUTHOR:	  	Scott Streit
*	DATE:     	12-16-1999
*	VERSION:  	1
*
*	DESCRIPTION:	This native method is used to check PCI DSP synthetically
*			created images for errors. Assumes bytewise differences.
*
*	PARAMETERS:	fd  		A reference to the image data that is to
*					be checked.
*			mem_length	The length, in words, of the image data
*					to check.
*
*	RETURNS:	The number of words successfully checked is returned. An
*			Exception is thrown if an incorrect data value is found.
*
*******************************************************************************/
#include <stdio.h>
#include <jni.h>
#include "MemorySetup.h"

enum boolean {false, true};

JNIEXPORT jint JNICALL
Java_MemorySetup_Cdata_1check(JNIEnv *env, jclass clazz, jint fd, jint mem_length)
{
	int first_value1 = 0;
	int first_value2 = 0;
	int first_value3 = 0;
	int first_value4 = 0;
	int second_value1 = 0;
	int second_value2 = 0;
	int second_value3 = 0;
	int second_value4 = 0;
	int first_incr = 0;
	int second_incr = 0;
	int third_incr = 0;
	int fourth_incr = 0;
	int word_count = 1;
	int data0 = 0;
	int data1 = 0;
	int data = 0;
	int prev_data = 0;
	int first_word;
	int second_word;
	int *addrPtr = (void *)fd;	
	int x;
	enum boolean recalculate_increment = 0;
	char message[100];
	jclass newExcCls;
	
	/* Read the first eight bytes. */
	first_word = addrPtr[0];
	second_word = addrPtr[1];

	first_value1 = (first_word & 0xffff);
	first_value2 = ((first_word & 0xffff0000) >> 16);
			
	second_value1 = (second_word & 0xffff);
	second_value2 = ((second_word & 0xffff0000) >> 16);

	first_incr = (second_value1 - first_value1) & 0xffff;
	second_incr = (second_value2 - first_value2) & 0xffff;
	
	/* Check the data using the increments. */
	for (x=2; x<mem_length; x++, word_count++) {
		first_word = addrPtr[x-1];
		second_word = addrPtr[x];

		first_value1 = (first_word & 0xffff);
		first_value2 = ((first_word & 0xffff0000) >> 16);

		second_value1 = (second_word & 0xffff);
		second_value2 = ((second_word & 0xffff0000) >> 16);

		/* Recalculate the increments on "roll-over" boundaries. */
		if (recalculate_increment == true) {
			first_incr = (second_value1 - first_value1) & 0xffff;
			second_incr = (second_value2 - first_value2) & 0xffff;
			recalculate_increment = false;
		}

		/* Check for data "roll-over". */
		if (first_value1 == 0xffff || first_value2 == 0xffff) {
			recalculate_increment = true;
		}
					
		/* If no data "roll-over", then check the increments. */
		else if (((second_value1 - first_value1) & 0xffff) != first_incr) {
			newExcCls = (*env)->FindClass(env, "java/lang/Exception");
       
       			/* Unable to find the new exception class, give up. */
       			if (newExcCls == 0)
       				return;

			sprintf(message, "	Expected: 0x%08X \n	Actual: 0x%08X \n	Successfully checked %d bytes"
			        , ((first_value1 + first_incr) & 0xffff), second_value1, (word_count/4));
       		
			(*env)->ThrowNew(env, newExcCls, message);
			break;
		}
		else if (((second_value2 - first_value2) & 0xffff) != second_incr) {
				newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
       			/* Unable to find the new exception class, give up. */
       			if (newExcCls == 0)
       				return;

			sprintf(message, "	Expected: 0x%08X \n	Actual: 0x%08X \n	Successfully checked %d bytes"
			        , ((first_value2 + second_incr) & 0xffff), second_value2, (word_count/4));
       		
			(*env)->ThrowNew(env, newExcCls, message);
			break;
		}
	}
	return word_count;
}
