/*******************************************************************************
*	FUNCTION: 	Cmap_memory.c
*	LIBRARY:  	mem
*	AUTHOR:	  	Scott Streit
*	DATE:     	03-15-2000
*	VERSION:  	1
*
*	DESCRIPTION:	This native method is used to memory map a kernel image
*			buffer directly to the user application.
*
*	PARAMETERS:	buffer_size  The size of the buffer (in bytes) to be
*				     allocated.
*			pci_fd	     A file descriptor pointing to the pci board.
*
*	RETURNS:	The start address of the mapped image buffer is returned.
*			An Exception is thrown if the image buffer cannot be
*			mapped.
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/mman.h>
#include "Cmemory.h"
#include "MemorySetup.h"

JNIEXPORT jint JNICALL Java_MemorySetup_Cmap_1memory(JNIEnv *env, jclass clazz, jint pci_fd, jint buffer_size)
{
	char *fd;
	int x;
	jclass newExcCls;
	char message[100];

	printf("\nCalling mmap ...\n");

	/* Map the kernel buffer */
	fd = mmap(0, buffer_size, (PROT_READ | PROT_WRITE), MAP_SHARED, pci_fd, 0);

	printf("\nmmap fd = 0x%X\n", fd);

	/* Check for an out of physical memory error. */
	if (fd == NULL && errno == ENOMEM) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		(*env)->ThrowNew(env, newExcCls, "Error Allocating Image Buffers. The Systems \n Physical Limits Have Been Exceeded. \n");
        }

	/* Check for an out of memory error. */
	else if (fd == NULL && errno == EAGAIN) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		(*env)->ThrowNew(env, newExcCls, "Error Allocating Image Buffers. \n Not Enough Memory CURRENTLY Available. \n");
        }
	
	/* Check for any other errors. */
	else if (fd == NULL) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		sprintf(message, "Error Allocating Memory (errno = %d) \n", errno);
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }

	return (int)fd;
}
