/*******************************************************************************
*	FUNCTION: 	Cioctl.c
*	LIBRARY:  	driver
*	AUTHOR:	  	Scott Streit
*	DATE:     	11-03-1999
*	VERSION:  	1
*
*	DESCRIPTION:	This native method is used to send control commands
*			to the camera and other special functions.
*
*	PARAMETERS:	pci_fd	     File descriptor returned from Copen.
*			Juin	     All input parameters.
*			pci_request  A valid ioctl command for the PCI board.
*
*	RETURNS:	The approprate value is returned (if there is a return
*			value) to the user through the Juin array. An Exception
*			is thrown if a problem occurs while transfering data
*			between the user and the device driver. In this case, a
*			"-1" and EFAULT are returned from ioctl. An Exception also
*			occurs if the PCI DSP FIFO is not ready to accept commands
*			and a set number of attempts fail. In this case, a "-1" and
*			EIO are returned from ioctl.
*******************************************************************************/
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <jni.h>
#include "Cmemory.h"
#include "MemorySetup.h"

#define SIZE	100

JNIEXPORT jintArray JNICALL
Java_MemorySetup_Craw2java(JNIEnv *env, jclass clazz, jint mem_fd, jintArray Juin, jint row)
{
	int i;

	/* Define reply buffer */
	int uin[SIZE];
	int *reply_arg = (int *)&uin[0];
  
  	addrPtr = (void *)mem_fd;

	/* Map the Java uin array to the C uin array. */
	(*env)->GetIntArrayRegion(env, Juin, 0, SIZE, uin);

	for (i=row*SIZE; i<row*SIZE+SIZE; i++)
		uin[i] = addrPtr[i];
  
	/* Map the C uin array back to the Java uin array. */
	(*env)->SetIntArrayRegion(env, Juin, 0, SIZE, uin);

	return Juin;
}
