/*******************************************************************************
*	FUNCTION: 	Cmap_memory.c
*	LIBRARY:  	mem
*	AUTHOR:	  	Scott Streit
*	DATE:     	03-15-2000
*	VERSION:  	1
*
*	DESCRIPTION:	This native method is used to unmap a memory mapped
*			kernel image buffer.
*
*	PARAMETERS:	buffer_size  The size of the buffer (in bytes) to be
*				     allocated.
*			map_fd	     A file descriptor pointing to the memory
*				     mapped region.
*
*	RETURNS:	None. An Exception is thrown if the image buffer cannot
*			be unmapped.
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/mman.h>
#include "Cmemory.h"
#include "MemorySetup.h"

JNIEXPORT void JNICALL Java_MemorySetup_Cunmap_1memory(JNIEnv *env, jclass clazz, jint map_fd, jint buffer_size)
{
	int value;
	jclass newExcCls;
	char message[100];

	printf("\nCalling munmap with 0x%X ...\n", map_fd);

	/* Map the kernel buffer */
	value = munmap((char *)map_fd, buffer_size);

	printf("\nmunmap value = %d\n", value);

	/* Check for any other errors. */
	if (value == -1) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		sprintf(message, "Error Allocating Memory (errno = %d) \n%s", errno, strerror(errno));
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }
}
