/**
********************************************************************************
*       This class handles the commands for the Camera buttons on the main
*	window.
* 	
*	CVM - Model
*
* 	@date June 28, 2000
	@see VoodooMainWindow
*	@version 1.00
*       @author Scott Streit 
********************************************************************************
*/
import java.io.*;
import java.util.*;
import javax.swing.*;

public class ArrayControlCommand extends SwingWorker
{
	private VoodooMainWindow parent;
	private DspCommand2 dsp_cmd;
	private String command = null; 

        /**
        ********************************************************************************
        *       Class constructor. 
        *
        *       @version 1.00
        *       @author Scott Streit 
        ********************************************************************************
        */
        public ArrayControlCommand(VoodooMainWindow parent) 
	{ 
		this.parent = parent;
		dsp_cmd = new DspCommand2();
	}

	/**
        ********************************************************************************
        *       This method initiates the command. It is automatically called by the
        *       SwingWorker class constructor.
        *
        *       @see SwingWorker
        *       @version 1.00
        *       @author Scott Streit 
        ********************************************************************************
        */ 
        public Object construct() 
        {
		return do_command();
        }  

	/**
        ********************************************************************************
        *       This method sets the command to be sent to the camera.
        *
        *       @version 1.00
        *       @author Scott Streit 
        ********************************************************************************
        */ 
        public void set_command(String cmd) 
        {
		command = cmd;
        }

        /**
        ********************************************************************************
        *       This method handles sending the commands for the Camera buttons.
        *
        *       @version 1.00
	*	@see SwingWorker
	*	@see DspCommand2
        *       @author Scott Streit 
        ********************************************************************************
        */
	public Object do_command() 
	{
		int reply_value = 0;

		if (command.equals("Open Shutter")) {
			try {
				PrintMessage.message("ArrayControlCommand", 0, "Opening shutter...waiting.\n", 0, true);
				dsp_cmd.hcvr(VoodooConstants.OPEN_SHUTTER, VoodooConstants.DON);
				parent.open_shutter_now_button.setText("Close Shutter");
				PrintMessage.message("ArrayControlCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: Could not open shutter.\n", 0, true);
				PrintMessage.message("ArrayControlCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
			}
			catch (Exception ex) {
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: " + ex.getMessage() + "\n", 0, true);
			}
		}
				
		else if (command.equals("Close Shutter")) {
			try {
				PrintMessage.message("ArrayControlCommand", 0, "Closing shutter...waiting.\n", 0, true);
				dsp_cmd.hcvr(VoodooConstants.CLOSE_SHUTTER, VoodooConstants.DON);
				parent.open_shutter_now_button.setText("Open Shutter");
				PrintMessage.message("ArrayControlCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: Could not close shutter.\n", 0, true);
				PrintMessage.message("ArrayControlCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
			}
			catch (Exception ex) {
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: " + ex.getMessage() + "\n", 0, true);
			}
		}		
		
		else if (command.equals("Clear Array")) {
			try {
				PrintMessage.message("ArrayControlCommand", 0, "Clearing array...waiting.\n", 0, true);
				parent.clear_array_button.setEnabled(false);
				dsp_cmd.hcvr(VoodooConstants.CLEAR_ARRAY, VoodooConstants.DON);
				parent.clear_array_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				parent.clear_array_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: Could not clear array.\n", 0, true);
				PrintMessage.message("ArrayControlCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
			}
			catch (Exception ex) {
				parent.clear_array_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: " + ex.getMessage() + "\n", 0, true);
			}
		}

		else if (command.equals("Idle Array")) {
			try {
				PrintMessage.message("SetupCommand", 0, "Setting camera to idle...waiting.\n", 0, true);
				dsp_cmd.idle(true);
				parent.idle_array_button.setText("Stop Idle");
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: Could not clear array.\n", 0, true);
				PrintMessage.message("ArrayControlCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
			}
			catch (Exception e) {
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			}
		}

		else if (command.equals("Stop Idle")) {
			try {
				PrintMessage.message("SetupCommand", 0, "Stopping camera idle...waiting.\n", 0, true);
				dsp_cmd.idle(false);
				parent.idle_array_button.setText("Idle Array");
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: Could not clear array.\n", 0, true);
				PrintMessage.message("ArrayControlCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
			}
			catch (Exception e) {
				if (abort) { return "Done"; }
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			}
		}

		else if (command.equals("Reset Controller")) {
			try {
				PrintMessage.message("ArrayControlCommand", 0, "Resetting controller...waiting.\n", 0, true);
				parent.reset_controller_button.setEnabled(false);
				dsp_cmd.hcvr(VoodooConstants.RESET_CONTROLLER, VoodooConstants.SYR);
				parent.reset_controller_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				parent.reset_controller_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: Could not reset controller.\n", 0, true);
				PrintMessage.message("ArrayControlCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.SYR) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
			}
			catch (Exception ex) {
				parent.reset_controller_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: " + ex.getMessage() + "\n", 0, true);
			}
		}

		else if (command.equals("Reset PCI")) {
			try {
				PrintMessage.message("ArrayControlCommand", 0, "Resetting PCI PC...waiting.\n", 0, true);
				parent.reset_pci_pc_button.setEnabled(false);
				dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
				parent.reset_pci_pc_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				parent.reset_pci_pc_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: Could not reset PCI PC.\n", 0, true);
				PrintMessage.message("ArrayControlCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ArrayControlCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
			}
			catch (Exception ex) {
				parent.reset_pci_pc_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 1, "failed.", 8, false);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: " + ex.getMessage() + "\n", 0, true);
			}
		}

		else if (command.equals("Read Temp")) {
			try {
				parent.read_temperature_button.setEnabled(false);
				parent.set_temperature_label(dsp_cmd.get_temperature());
				parent.read_temperature_button.setEnabled(true);
			}
			catch (ReplyValueException rve) {
				parent.read_temperature_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: Could not read array temperature.\n", 0, true);
			}
			catch (Exception ex) {
				parent.read_temperature_button.setEnabled(true);
				PrintMessage.message("ArrayControlCommand", 1, "ERROR: " + ex.getMessage() + "\n", 0, true);
			}		
		}

		return "Done";
	}
}
