/**
********************************************************************************
*       This class sends the Binning ControllerConfig information to the PCI
*			interface board.
* 	
*	CVM - Model
*
* 	@filename ControllerConfigCommand.java
* 	@date Summer 2000
*	@version 1.00
*       @author Xiangyang (Helena) Xian
********************************************************************************
*/
import java.io.*;
import java.util.*;
import javax.swing.*;

public class ControllerConfigCommand extends SwingWorker
{
	protected ControllerConfigDialog parent;
	private DspCommand2 dsp_cmd;
	private ManualCommand manual_cmd;
	private boolean do_binning;
	private boolean do_readout;
	private boolean do_serial_readout;
	private boolean do_both_readouts;
	private boolean do_mpp;
	private boolean do_coadder;
	private boolean do_coadd_download;
	private boolean do_number_of_coadds;
	private boolean do_gain;
	private boolean do_video_offsets;
	private boolean do_temperature;
	private boolean do_all;
	private boolean binning_selected;
	private boolean readout_selected;
	private boolean mpp_selected;
	private boolean coadder_selected;
	private boolean gain_selected;
	private boolean video_offsets_selected;
	private boolean temperature_selected;
	private double temperature;
	private int mpp_value;
	private int amp;
	private int value_of_gain;
	private int value_of_speed;
	private int number_of_coadds;
	private int binning_parameters_bx;
	private int binning_parameters_by;
	private int user_input[] = new int[32];

        /**
        ********************************************************************************
        *       Class constructor. No object is instantiated. 
        *
        *       @version 1.00
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
        public ControllerConfigCommand(ControllerConfigDialog parent)
        {
        	this.parent  = parent;

		do_binning = false;
		do_readout = false;
		do_serial_readout = false;
		do_both_readouts = false;
		do_mpp = false;
		do_coadder = false;
		do_coadd_download = false;
		do_number_of_coadds = false;
		do_gain = false;
		do_video_offsets = false;
		do_all = false;

		binning_selected = false;
		readout_selected = false;
		mpp_selected = false;
		coadder_selected = false;
		gain_selected = false;
		video_offsets_selected = false;
		temperature_selected = false;
		
		amp = 0;
		mpp_value = 0;
		value_of_gain = 0;
		value_of_speed = 0;
		number_of_coadds = 0;
		temperature = 0.0;
		binning_parameters_bx = 1;
		binning_parameters_by = 1;
        }

	/**
        ********************************************************************************
        *       This method initiates the download. It is automatically called by the
        *       SwingWorker class constructor.
        *
        *       @see SwingWorker
	* 	@see DspCommand2
        *       @version 1.00
        *       @author Xiangyang (Helena) Xian 
        ********************************************************************************
        */ 
        public Object construct() 
        {
		return apply();	
        }  

        /**
        ********************************************************************************
        *       This method sets the "Apply" button back to "Apply". This method is
        *       		automatically called by the SwingWorker class.
        *
        *       @see SwingWorker
        *       @version 1.00
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
        public void finished()
        {
		parent.set_apply_button_text("Apply Above");
		parent.set_apply_all_button_text("Apply All");
		dsp_cmd.abort = false;
		dsp_cmd = null;

		// Re-initialize all selected values to false for next time. Otherwise,
		// you may have controller properties executing that you don't want to.
		binning_selected = false;
		readout_selected = false;
		mpp_selected = false;
		coadder_selected = false;
		gain_selected = false;
		video_offsets_selected = false;
		temperature_selected = false;
		do_all = false;
        }

        /**
        ********************************************************************************
        *       This method aborts the current thread. This method overides the one in
        *       		Swingworker.java.
        *
        *       @see SwingWorker
        *       @version 1.00
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
        public void stop()
        {
                abort = true;
                dsp_cmd.abort = true;

		// Reset the PCI board DSP program counter.
		try {
			dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
		}
		catch(ReplyValueException RVE) {}
		catch(Exception E) {}		
        }


        /**
        ********************************************************************************
        *	This method sets the binning to on/off.
	*
	*	@param binning_on True to perform the binning on the controller, false to
	*			not perform the binning on the controller.
	*
	* 	@version 1.00 
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
	public void set_binning(boolean binning_on)
	{
		do_binning = binning_on;
	}

	/**
        ********************************************************************************
        *	This method sets the coadder to on/off.
	*	
	*	@param coadder_on True to perform the coadder on the controller,
	*			false to not perform the coadder on the controller.
	*
	*	@version 1.00
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
	public void set_coadder(boolean coadd)
	{
		do_coadder = coadd;
	}

	/**
        ********************************************************************************
        *	This method sets the coadder download to on/off.
	*	
	*	@param coadder_on True to download the coadder DSP code to the controller,
	*			  false to not download the code.
	*
	*	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void set_coadd_download(boolean coadd_download)
	{
		this.do_coadd_download = coadd_download;
	}

	/**
	********************************************************************************
	*	This method sets the number of coadds to perform.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_number_of_coadds(boolean do_number_of_coadds)
	{
		this.do_number_of_coadds = do_number_of_coadds;
	}

	/**
        ********************************************************************************
        *	This method sets the gain to on/off.
	*	
	*	@param gain_on True to perform the gain on the controller,
	*			false to not perform the gain on the controller.
	*
	*	@version 1.00
        *       @author Xiangyang "Helena" Xian
        ********************************************************************************
        */
	public void set_gain(boolean gain_on)
	{
		do_gain = gain_on;
	}

	/**
        ********************************************************************************
        *	This method sets the video offsets to on/off.
	*	
	*	@param video_offsets_on True to perform the video offsets on the
	*				controller,
false to not perform the video
	*				offsets on the controller.
	*
	*	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void set_video_offsets(boolean video_offsets_on)
	{
		do_video_offsets = video_offsets_on;
	}

        /**
        ********************************************************************************
        *	This method specifies whether or not any readout commands are implemented.
	*
	*	@param readout True if set readout mode is implemented,	false if it is
	*		       not.
	*
	* 	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void set_readout(boolean readout)
	{
		 do_readout = readout;
	}

        /**
        ********************************************************************************
        *	This method sets the split_serial commmand to on/off.
	*
	*	@param split_serial_on True to perform the split_serial on the controller, 
	*			false to not perform the split_serial on the controller.
	*
	* 	@version 1.00
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
	public void set_serial_readout(boolean serial_readout)
	{
		 do_serial_readout = serial_readout;
	}

	/**
        ********************************************************************************
        *	This method sets the split_parallel commmand to on/off.
	*
	*	@param split_parellel_on True to perform the split_parallel on the controller, 
	*			false to not perform the split_parallel on the controller.
	*
	* 	@version 1.00
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
	public void set_both_readouts(boolean both_readouts)
	{
		 do_both_readouts = both_readouts;
	}

	/**
	********************************************************************************
	*	This method sets the MPP to on/off.
	*	
	*	@param mpp_on True to use MPP,
False to not use MPP. 
	*
	*	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void set_mpp(boolean mpp_on)
	{
		do_mpp = mpp_on;
	}

	/**
        ********************************************************************************
        *	This method sets the temperature to on/off.
	*	
	*	@param temperature_on True to perform the temperature on the controller,
	*			false to not perform the temperature on the controller.
	*
	*	@version 1.00
        *       @author Xiangyang "Helena" Xian
        ********************************************************************************
        */
	public void set_temperature(boolean temperature_on)
	{
		do_temperature = temperature_on;
	}

        /**
        ********************************************************************************
        *	This method selects binning for current command issue.
	*
	*	@param binning True to send the binning to the controller, false to
	*			not send the binning to the controller.
	*
	* 	@version 1.00 
        *       @author Scott Streit
        ********************************************************************************
        */
	public void select_binning(boolean binning)
	{
		binning_selected = binning;
	}

	/**
        ********************************************************************************
        *	This method selects coadder for current command issue.
	*	
	*	@param coadder True to send the coadder to the controller,
	*			false to not send the coadder to the controller.
	*
	*	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void select_coadder(boolean coadd)
	{
		coadder_selected = coadd;
	}

	/**
        ********************************************************************************
        *	This method selects gain for current command issue.
	*	
	*	@param gain True to send the gain to the controller,
	*			false to not send the gain to the controller.
	*
	*	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void select_gain(boolean gain)
	{
		gain_selected = gain;
	}

	/**
        ********************************************************************************
        *	This method selects video offsets for current command issue.
	*	
	*	@param video_offsets True to send the video offsets to the controller,
	*			false to not send the video offsets to the controller.
	*
	*	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void select_video_offsets(boolean video_offsets)
	{
		video_offsets_selected = video_offsets;
	}

        /**
        ********************************************************************************
        *	This method selects readout for current command issue.
	*
	*	@param readout True to send readout info to the controller, false to
	*			not send any readout info the controller.
	*
	* 	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void select_readout(boolean readout)
	{
		 readout_selected = readout;
	}

	/**
	********************************************************************************
        *	This method selects MPP for current command issue.
	*	
	*	@param temperature True to send the MPP to the controller,
	*			false to not send the MPP to the controller.
	*
	*	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void select_mpp(boolean mpp)
	{
		mpp_selected = mpp;
	}

	/**
        ********************************************************************************
        *	This method selects temperature for current command issue.
	*	
	*	@param temperature True to send the temperature to the controller,
	*			false to not send the temperature to the controller.
	*
	*	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void select_temperature(boolean temperature)
	{
		temperature_selected = temperature;
	}

	/**
	********************************************************************************
	*	This method sets the apply() method to apply all configurations.
	*	
	*	@param all_on True to apply all configurations,
False to apply only a
	*		      specific one. 
	*
	*	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
	public void set_all(boolean all_on)
	{
		do_all = all_on;
	}

	/**
	********************************************************************************
	*	This method sets the amplifier(s) to use for readout.
	*
	*	@param amp The amplifier to use. Depending on the mode, may be one of
	*		   the following ascii strings: "__A", "__B", "__C", "__D",
	*		   "_AB", "_CD", "__L", "__R", "_LR", or "ALL".
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_amplifier(int amp)
	{
		this.amp = amp;
	}

	/**
	********************************************************************************
	*	This method sets the MPP value.
	*
	*	@param mpp_value The MPP value. May be 0 for no MPP mode or 1 for MPP.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_mpp_value(int mpp_value)
	{
		this.mpp_value = mpp_value;
	}

	/**
	********************************************************************************
	*	This method sets the value for binning_cols according to what the user
	* 			chooses.
	* 	
	* 	@param bx binning parameters columns.
	*
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian
	********************************************************************************
	*/
	public void set_binning_cols(int bx)
	{
		binning_parameters_bx = bx;
	}
  
        /**
        ********************************************************************************
        *       This method sets the value for binning_rows according to what the user
        *	chooses.
        *
        *       @param by binning paremeters rows.
	*
	*	@version 1.00
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
	public void set_binning_rows(int by)
	{
		binning_parameters_by = by;
	}

	/**
	********************************************************************************
	*	This method sets the video gain.
	*
	*	@param gain The gain value.
	*
	*	@version 1.00
	*	@author Xiangyang "Helena" Xian
	********************************************************************************
	*/
	public void set_gain(int gain)
	{
		value_of_gain = gain;
	}


	/**
	********************************************************************************
	*	This method sets the integrator speed.
	*
	*	@param speed The integrator speed.
	*
	*	@version 1.00
	*	@author Xiangyang "Helena" Xian
	********************************************************************************
	*/
	public void set_speed(int speed)
	{
		value_of_speed = speed;
	}

        /**
        ********************************************************************************
        *	This method parses and sets the video offsets. The input strings are
        *	converted to integers.
	*
	*	@param video_offset The video offset value.
	*	@param index The video offset number.
	*
	*	@throws NumberFormatException If the video offset value is out of range
	*			or could not be parsed.
	*
	* 	@version 1.00 
        *       @author Scott Streit
        ********************************************************************************
        */
        public void set_video_offsets(int index, String video_offset) throws NumberFormatException
        {
		if (do_video_offsets && (video_offsets_selected || do_all)) {
			try {
				user_input[index] = Integer.parseInt(video_offset);

				if ((user_input[index] < 0) || (user_input[index] > 4095))
					throw new NumberFormatException("ERROR: Video offset #" + Integer.toString(index) + " out of range(0-4095)");
			}
			catch (NumberFormatException nfe) {
				throw new NumberFormatException("ERROR: Cannot parse video offset #" + Integer.toString(index));
			}
		}
	}

        /**
        ********************************************************************************
        *	This method parses and sets the number of coadds to perform. The input
        *	string is converted to an integer.
	*
	*	@param coadds_num The number of coadds to perform.
	*
	*	@throws NumberFormatException If the number of coadds is out of range
	*			or could not be parsed.
	* 	@version 1.00 
        *       @author Scott Streit
        ********************************************************************************
        */
        public void set_number_of_coadds(String coadds_num) throws NumberFormatException
        {
	       	if (do_coadder && (coadder_selected || do_all)) {
			try {
				number_of_coadds = Integer.parseInt(coadds_num);

				if ((number_of_coadds < 0) || (number_of_coadds > 65536))
					throw new NumberFormatException("ERROR: Invalid number of coadds (0-65536)");
			}
			catch (NumberFormatException nfe) {
				throw new NumberFormatException("ERROR: Cannot parse coadd textfield");
			}
		}
	}

        /**
        ********************************************************************************
        *	This method parses and sets the array temperature. The input string is
        *	converted to an double precision value.
	*
	*	@param desired_temperature The lower limit of the array temperature.
	*
	*	@throws NumberFormatException If the temperature could not be parsed.
	*
	* 	@version 1.00 
        *       @author Scott Streit
        ********************************************************************************
        */
        public void set_temperature(String desired_temperature) throws NumberFormatException
        {
       		if (do_temperature && (temperature_selected || do_all)) {
			try {
				temperature = Double.parseDouble(desired_temperature);
			}
			catch (NumberFormatException nfe) {
				throw new NumberFormatException("ERROR: Cannot parse temperature textfield");
			}
		}
	}

        /**
        ********************************************************************************
        *       This method sends all the ControllerConfig information to the PCI 
	*	interface board.
        *
        *	IMPORTANT NOTE: The ManualCommand Class is NOT called through the
        *			thread_it() method on purpose. It is less time costly
        *			and more code compact to NOT instantiate a new thread
        *			for every configuration. Instead the ManualCommand
        *			apply() method is called directly. Also, this command
        *			class is already in a thread. If, however, one should
        *			decide differently later, here is a useful wait method:
	*
        *			private void thread_wait()
        *			{
        *				while(manual_cmd.alive()) {
        *					try
        *						Thread.sleep(5);
        *					catch (InterruptedException ie) {}
        *				}
        *			}
        *
        *       @version 1.00
	*	@see SwingWorker
	*	@see DspCommand2
        *       @author Xiangyang (Helena) Xian 
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	07/28/2000	sds	Replaced all ManualCommand thread_it()'s with
	*				apply(). Fixed up coadder section. Modified to
	*				use "Apply" and "Apply All" buttons.
        ********************************************************************************
        */
	public Object apply() 
	{
		int reply_value = 0;
		dsp_cmd = new DspCommand2();
		manual_cmd = new ManualCommand();
		abort = false;

		if (abort) { return "Done"; }

		if (do_gain && (gain_selected || do_all) && !abort) {
			PrintMessage.message("ControllerConfigCommand", 0, "Setting gain and speed...\n", 0, true);
			manual_cmd.set_board_destination(VoodooConstants.TIM_ID);
			manual_cmd.set_arg1(value_of_gain);
			manual_cmd.set_arg2(value_of_speed);
			manual_cmd.set_command(VoodooConstants.SGN);
			manual_cmd.apply();
			manual_cmd.finished();
		}

		if (do_video_offsets && (video_offsets_selected || do_all) && !abort) {
			for (int i = 0; i < 32; i+=2) {

				if (abort) { return "ERROR"; }

				// For each of the odd channels.
     				if (parent.channel_txtfld[i].isEnabled()) {
          				PrintMessage.message("ControllerConfigCommand", 0, "Setting video offset...\n", 0, true);
					manual_cmd.set_board_destination(VoodooConstants.TIM_ID);
					manual_cmd.set_arg1(i/2);
					manual_cmd.set_arg2(1);
					manual_cmd.set_arg3(VoodooConstants.VID);
					manual_cmd.set_arg4(user_input[i]);
					manual_cmd.set_command(VoodooConstants.SBN);
					manual_cmd.apply();
					manual_cmd.finished();
				}

				

				if (abort) { return "ERROR"; }

				
               			// For each of the even channels.
                  		if (parent.channel_txtfld[i+1].isEnabled()) {
					PrintMessage.message("ControllerConfigCommand", 0, "Setting video offset...\n", 0, true);
					manual_cmd.set_board_destination(VoodooConstants.TIM_ID);
					manual_cmd.set_arg1(i/2);
					manual_cmd.set_arg2(3);
					manual_cmd.set_arg3(VoodooConstants.VID);
					manual_cmd.set_arg4(user_input[i+1]);
					manual_cmd.set_command(VoodooConstants.SBN);
					manual_cmd.apply();
					manual_cmd.finished();
      				}
			}  
		}

		if (do_coadder && (coadder_selected || do_all) && !abort) {
			manual_cmd.set_board_destination(VoodooConstants.TIM_ID);

			if (do_coadd_download) {
				PrintMessage.message("ControllerConfigCommand", 0, "Setting coadder download...\n", 0, true);
				manual_cmd.set_command(VoodooConstants.DCA);
				manual_cmd.apply();
				manual_cmd.finished();
			}
			
			if (do_number_of_coadds) {
				PrintMessage.message("ControllerConfigCommand", 0, "Setting number of coadds...\n", 0, true);
				manual_cmd.set_arg1(number_of_coadds);
				manual_cmd.set_command(VoodooConstants.SNC);
				manual_cmd.apply();
				manual_cmd.finished();
			}
		}	


		if (do_temperature && (temperature_selected || do_all) && !abort) {
			try {
				PrintMessage.message("ControllerConfigCommand", 0, "Setting array temperature...waiting.\n", 0, true);
				dsp_cmd.set_temperature(temperature);
				PrintMessage.message("ControllerConfigCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				if (abort) { return "Done"; }
				PrintMessage.message("ControllerConfigCommand", 1, "failed.", 8, false);
				return "ERROR";
			}
			catch (Exception e) {
				if (abort) { return "Done"; }
				PrintMessage.message("ControllerConfigCommand", 1, "failed.", 8, false);
				PrintMessage.message("ControllerConfigCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				return "ERROR";
			}
		}

		if (do_binning && (binning_selected || do_all) && !abort) { 
			try {
				PrintMessage.message("ControllerConfigCommand", 0, "Setting binning parameters...waiting.\n", 0, true);
				dsp_cmd.set_board_destination(VoodooConstants.WRM_ARG_NUM, VoodooConstants.TIM_ID);
				dsp_cmd.set_type(VoodooConstants.Y);
				dsp_cmd.write_memory(0x5, binning_parameters_bx);
				dsp_cmd.write_memory(0x6, binning_parameters_by);
				PrintMessage.message("ControllerConfigCommand", 0, "done.", 8, false);
			}
			catch(ReplyValueException rve) {
				if (abort) { return "Done"; }
				reply_value = rve.get_value();
				PrintMessage.message("ControllerConfigCommand", 1, "failed.", 8, false);
			}
			catch(Exception e) {
				if (abort) { return "Done"; }
				PrintMessage.message("ControllerConfigCommand", 1, "failed.", 8, false);
			}
		} 

		if (do_both_readouts && (readout_selected || do_all) && !abort) {
			PrintMessage.message("ControllerConfigCommand", 0, "Setting Readout mode...\n", 0, true);
			manual_cmd.set_board_destination(VoodooConstants.TIM_ID);
			manual_cmd.set_arg1(amp);
			manual_cmd.set_command(VoodooConstants.SOS);
			manual_cmd.apply();
			manual_cmd.finished();
		}

		else if (do_serial_readout && (readout_selected || do_all) && !abort) {
			PrintMessage.message("ControllerConfigCommand", 0, "Setting readout mode...\n", 0, true);
			manual_cmd.set_board_destination(VoodooConstants.TIM_ID);
			manual_cmd.set_arg1(amp);
			manual_cmd.set_command(VoodooConstants.SOS);
			manual_cmd.apply();
			manual_cmd.finished();
		}

		if (do_mpp && (mpp_selected || do_all) && !abort) {
			PrintMessage.message("ControllerConfigCommand", 0, "Setting MPP mode....\n", 0, true);
			manual_cmd.set_board_destination(VoodooConstants.TIM_ID);
			manual_cmd.set_arg1(mpp_value);
			manual_cmd.set_command(VoodooConstants.MPP);
			manual_cmd.apply();
			manual_cmd.finished();
		}

		return "Done";
	}
}
   
