/**
********************************************************************************
*       This class is the event listener for the ControllerConfigDialog window. 
*	All actions handled by this class are passed to the ControllerConfigCommand
* 	class.
*	
*	CVM - Controller
*
* 	@filename ControllerConfigListener.java
* 	@date Summer 2000
*	@version 1.00
*	@author Xiangyang (Helena) Xian
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;
import java.io.*;

public class ControllerConfigListener implements ItemListener, ActionListener, ChangeListener
{
	protected ControllerConfigCommand controller_config_cmd = null;
        protected int derived_size_x = 0;
        protected int derived_size_y = 0;
	protected int array_mode;
	private ControllerConfigDialog parent = null;
	private int selectedIndex;
	private String message;
	
        /**
        ********************************************************************************
        *       Class constructor.
        *
        *       @param parent The dialog window that this class listens for.
        *
        *       @version 1.00
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
        public ControllerConfigListener(ControllerConfigDialog parent)
	{
		this.parent = parent;
		controller_config_cmd = new ControllerConfigCommand(parent);
		selectedIndex = -1;
		array_mode = 0;
        }

	/**
	*****************************************************************************
	*	This method handles item selection events from the Contoller Config
	*	Dialog window.
	*
	*	@param ie The selection event.
	*
	*	@see ItemListener
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	07/28/2000	sds	Coelesed all "Apply" button events into one
	*				master.	Removed the "Auto Deinterlace" option.
	*				Fixed up coadder code.
	*****************************************************************************
	*/	
	public void itemStateChanged(ItemEvent ie)
	{
		// Determine if the source is a JComboBox or a JCheckBox.
		if (ie.getItem().getClass().getName().equals("java.lang.String")) {
			JComboBox active_value = (JComboBox)ie.getSource();
			int index = 0;

			if (active_value.getActionCommand().equals("Column Binning")) {
				index = parent.binning_parameters_combo_Bx.getSelectedIndex()+1;
			
				// Set column binning value in command class.
				controller_config_cmd.set_binning_cols(index);

				// Set the derived array size textfield.
				derived_size_x = parent.image_ncols * index;
				parent.derived_array_size_x_txtfld.setText(Integer.toString(derived_size_x));
			}	
			
			else if (active_value.getActionCommand().equals("Row Binning")) {
				index = parent.binning_parameters_combo_By.getSelectedIndex()+1;
			
				// Set row binning value in command class.
				controller_config_cmd.set_binning_rows(index);

				// Set the derived array size textfield.
               			derived_size_y = parent.image_nrows * index;
                      		parent.derived_array_size_y_txtfld.setText(Integer.toString(derived_size_y));
			}

			else if (active_value.getActionCommand().equals("Video Offsets")) {
				int channel_index = parent.number_of_readout_channels_combo_box.getSelectedIndex()+1;

				// Enable the channel labels and the textfields of video offsets which have been selected. 
				// And make them visible.
				for (int j = 0; j < channel_index; j++) {
					parent.channel_label[j].setVisible(true);
					parent.channel_txtfld[j].setEnabled(true);
					parent.channel_txtfld[j].setVisible(true);
				}
				// Disable the channel labels and the textfields of video offsets which have not been selected.
				// And make them invisible.
				for (int k = channel_index; k < parent.channel_txtfld.length; k++) {
					parent.channel_label[k].setVisible(false);
					parent.channel_txtfld[k].setEnabled(false);
					parent.channel_txtfld[k].setVisible(false);
					parent.channel_txtfld[k].setText("0");
				}
			}
		}

		// Must be a JCheckBox.

		else {
			AbstractButton active_button = (AbstractButton) ie.getSource();

			// Set the array temperature.
			if (active_button.getText().equals("Array Temperature: ")) {
				if (ie.getStateChange() == ItemEvent.SELECTED)
					controller_config_cmd.set_temperature(true);
				else
					controller_config_cmd.set_temperature(false);
			}

			// Set the number of coadds per readout.
			else if (active_button.getText().equals("Set Number of Coadds per Readout")) {
				if (parent.number_of_coadder_chkbox.isSelected())
					controller_config_cmd.set_number_of_coadds(true);
				else
					controller_config_cmd.set_number_of_coadds(false);
			}

			// Download the coadder board DSP code.
			else if (active_button.getText().equals("Download Coadder Board DSP Code")) {
				if (parent.download_coadder_board_chkbox.isSelected())
					controller_config_cmd.set_coadd_download(true);
				else
					controller_config_cmd.set_coadd_download(false);
			}

			// Set the controller MPP mode.
			else if (active_button.getText().equals("Use MPP mode")) {
				if (parent.mpp_chkbox.isSelected())
					controller_config_cmd.set_mpp_value(1);
				else
					controller_config_cmd.set_mpp_value(0);	
			}
		}
	}

        /**
        ********************************************************************************
        *       This method handles all the Listener events.
        *
        *       @param evt The event.
        *
        *       @see ActionListener
        *       @version 1.00
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
        public void actionPerformed(ActionEvent evt)
	{
		AbstractButton active_button = (AbstractButton) evt.getSource();

		// Close the ControllerConfigDialog window.
		if (active_button.getText().equals("Close")) {
			parent.setVisible(false);
		}

		else if (active_button.getText().equals("Apply Above")) {
			// Set the apply button text to "Abort".
			parent.set_apply_button_text("Abort");

			// Set the configuration to apply.
			try {
				set_selected_configuration(parent.controller_config_tabbed_pane.getSelectedIndex());
			}
			catch (NumberFormatException nfe) {
				PrintMessage.message("ControllerConfigListener", 1, nfe.getMessage() + ".\n", 0, true);
				parent.set_apply_button_text("Apply Above");
				return;
			}
			catch (Exception exp) {
				PrintMessage.message("ControllerConfigListener", 1, exp.getMessage() + ".\n", 0, true);
				parent.set_apply_button_text("Apply Above");
				return;
			}
						
			// Apply the current controller settings.
			controller_config_cmd.thread_it();
		}

		else if (active_button.getText().equals("Abort")) {
			controller_config_cmd.stop();
			controller_config_cmd.interrupt();
			System.gc();
			
			// Set the apply button text to "Apply".
			parent.set_apply_button_text("Apply Above");
		} 


		else if (active_button.getText().equals("Apply All")) {
			// Set the apply button text to "Abort".
			parent.set_apply_all_button_text("Abort");

			// Set the configuration to apply.
			controller_config_cmd.set_all(true);

			// Check and set all textfield values.
			try {
				if (parent.nc_txtfld != null)
					controller_config_cmd.set_number_of_coadds(parent.nc_txtfld.getText());
				
				if (parent.temperature_txtfld != null)
					controller_config_cmd.set_temperature(parent.temperature_txtfld.getText());
				
				if (parent.channel_txtfld[0] != null) {
					for (int i = 0; i < parent.channel_txtfld.length; i++)
						controller_config_cmd.set_video_offsets(i, parent.channel_txtfld[i].getText());
				}

			}
			catch (NumberFormatException nfe) {
				PrintMessage.message("ControllerConfigListener", 1, nfe.getMessage() + ".\n", 0, true);
				parent.set_apply_all_button_text("Apply All");
				return;
			}
			catch (Exception exp) {
				PrintMessage.message("ControllerConfigListener", 1, exp.getMessage() + ".\n", 0, true);
				parent.set_apply_all_button_text("Apply All");
				return;
			}
			
			// Apply the current controller settings.
			controller_config_cmd.thread_it();
		}


		else if (active_button.getText().equals("Abort All")) {
			controller_config_cmd.stop();
			controller_config_cmd.interrupt();
			System.gc();
			
			// Set the apply button text to "Apply All".
			parent.set_apply_all_button_text("Apply All");
		} 

		else if (active_button.getText().equals("A")) {
			if (parent.single_readout_A_rbutton.isSelected()) { 
				parent.both_readout_pics_label.setIcon(parent.single_readout_A_icon);
				parent.readout_status_txtfld.setText("Single readout.");
				parent.readout_status_serial_txtfld.setText("No");
				parent.readout_status_parallel_txtfld.setText("No");
				controller_config_cmd.set_amplifier(VoodooConstants.A);
				parent.parent.parent.deinterlace_combo_box.setSelectedIndex(0);
			}
		}
	
		else if (active_button.getText().equals("B")) {
			if (parent.single_readout_B_rbutton.isSelected()) {
				parent.both_readout_pics_label.setIcon(parent.single_readout_B_icon);
				parent.readout_status_txtfld.setText("Single readout.");
				parent.readout_status_serial_txtfld.setText("No");
				parent.readout_status_parallel_txtfld.setText("No");
				controller_config_cmd.set_amplifier(VoodooConstants.B);
				parent.parent.parent.deinterlace_combo_box.setSelectedIndex(0);
			}		
		}
	
		else if (active_button.getText().equals("C")) {
			if (parent.single_readout_C_rbutton.isSelected()) {
				parent.both_readout_pics_label.setIcon(parent.single_readout_C_icon);
				parent.readout_status_txtfld.setText("Single readout.");
				parent.readout_status_serial_txtfld.setText("No");
				parent.readout_status_parallel_txtfld.setText("No");
				controller_config_cmd.set_amplifier(VoodooConstants.C);
				parent.parent.parent.deinterlace_combo_box.setSelectedIndex(0);
			}
		}
		
		else if (active_button.getText().equals("D")) {
			if (parent.single_readout_D_rbutton.isSelected()) {
				parent.both_readout_pics_label.setIcon(parent.single_readout_D_icon);
				parent.readout_status_txtfld.setText("Single readout.");
				parent.readout_status_serial_txtfld.setText("No");
				parent.readout_status_parallel_txtfld.setText("No");
				controller_config_cmd.set_amplifier(VoodooConstants.D);
				parent.parent.parent.deinterlace_combo_box.setSelectedIndex(0);
			}
		}
	
		if (active_button.getText().equals("AB")) {
			if (parent.dual_readout_AB_rbutton.isSelected()) {
				parent.both_readout_pics_label.setIcon(parent.dual_readout_AB_icon);
				parent.readout_status_txtfld.setText("Dual readout.");
				parent.readout_status_serial_txtfld.setText("Yes, split serial up.");
				parent.readout_status_parallel_txtfld.setText("No");
				controller_config_cmd.set_amplifier(VoodooConstants.AB);
				parent.parent.parent.deinterlace_combo_box.setSelectedIndex(2);
			}
		}

		else if (active_button.getText().equals("CD")) {
			if (parent.dual_readout_CD_rbutton.isSelected()) {
				parent.both_readout_pics_label.setIcon(parent.dual_readout_CD_icon);
				parent.readout_status_txtfld.setText("Dual Readout.");
				parent.readout_status_serial_txtfld.setText("Yes, split serial down.");
				parent.readout_status_parallel_txtfld.setText("No");
				controller_config_cmd.set_amplifier(VoodooConstants.CD);
				parent.parent.parent.deinterlace_combo_box.setSelectedIndex(2);
			}
		}

		else if (active_button.getText().equals("ALL")) {
			if (parent.quad_readout_ABCD_rbutton.isSelected()) {		
				parent.both_readout_pics_label.setIcon(parent.quad_readout_icon);
				parent.readout_status_txtfld.setText("Quad readout.");
				parent.readout_status_serial_txtfld.setText("Yes");
				parent.readout_status_parallel_txtfld.setText("Yes");
				controller_config_cmd.set_amplifier(VoodooConstants.ALL);
				
				// Set the deinterlace value.
				if (array_mode == VoodooConstants.CCD)
					parent.parent.parent.deinterlace_combo_box.setSelectedIndex(3);
				else
					parent.parent.parent.deinterlace_combo_box.setSelectedIndex(4);
			}
		}

		else if (active_button.getText().equals("LEFT")) {
			if (parent.serial_readout_L_rbutton.isSelected()) {
				parent.serial_readout_pics_label.setIcon(parent.dual_readout_serial_L_icon);
				parent.readout_status_txtfld.setText("One Readout on left.");
				parent.readout_status_serial_txtfld.setText("No");
				parent.readout_status_parallel_txtfld.setText("No");
				controller_config_cmd.set_amplifier(VoodooConstants.L);
				parent.parent.parent.deinterlace_combo_box.setSelectedIndex(0);
			}
		}
	
		else if (active_button.getText().equals("RIGHT")) {
			if (parent.serial_readout_R_rbutton.isSelected()) {
				parent.serial_readout_pics_label.setIcon(parent.dual_readout_serial_R_icon);
				parent.readout_status_txtfld.setText("One readout on right.");
				parent.readout_status_serial_txtfld.setText("No");
				parent.readout_status_parallel_txtfld.setText("No");
				controller_config_cmd.set_amplifier(VoodooConstants.R);
				parent.parent.parent.deinterlace_combo_box.setSelectedIndex(0);
			}
		}

		else if (active_button.getActionCommand().equals("LEFT/RIGHT")) {
			if (active_button.getText().equals("LEFT/RIGHT")) {
				if (parent.serial_readout_LR_rbutton.isSelected()) {
					parent.serial_readout_pics_label.setIcon(parent.dual_readout_serial_LR_icon);
					parent.readout_status_txtfld.setText("Two readout on both left and right.");
					parent.readout_status_serial_txtfld.setText("Yes");
					parent.readout_status_parallel_txtfld.setText("No");
					controller_config_cmd.set_amplifier(VoodooConstants.LR);
					parent.parent.parent.deinterlace_combo_box.setSelectedIndex(2);
				}
			}
		}

		// Set the controller gain to 1.0.
		else if (active_button.getText().equals("1.0")) {
			if (parent.gain_1_rbutton.isSelected())
				controller_config_cmd.set_gain(VoodooConstants.ONE);
		}
		
		// Set the controller gain to 2.0.
		else if (active_button.getText().equals("2.0")) {
			if (parent.gain_2_rbutton.isSelected())
				controller_config_cmd.set_gain(VoodooConstants.TWO);
		}
	
		// Set the controller gain to 4.75.
		else if (active_button.getText().equals("4.75")) {
			if (parent.gain_475_rbutton.isSelected())
				controller_config_cmd.set_gain(VoodooConstants.FIVE);
		}
	
		// Set the controller gain to 9.5.
		else if (active_button.getText().equals("9.5")) {
			if (parent.gain_95_rbutton.isSelected())
				controller_config_cmd.set_gain(VoodooConstants.TEN);
		}

		// Set the controller speed to slow.
		else if (active_button.getText().equals("Slow")) {
			if (parent.slow_speed_rbutton.isSelected())
				controller_config_cmd.set_speed(VoodooConstants.SLOW);
		}
		
		// Set the controller gain to fast.
		else if (active_button.getText().equals("Fast")) {
			if (parent.fast_speed_rbutton.isSelected())
				controller_config_cmd.set_speed(VoodooConstants.FAST);	
		}
	}
        
        /**
        *****************************************************************************
        *       Controls the switching of the "tabbed panes". This method
        *       overides the same method in the ChangeListener interface.
        *
        *       @see ChangeListener
        *       @version 1.00
        *       @author Scott Streit
        *****************************************************************************
        */
        public void stateChanged(ChangeEvent ce)
        {
                JTabbedPane tp = (JTabbedPane)ce.getSource();

                // Check if a tab has been selected for the first time or if a new tab
                // has been selected and set the tab to "false".
                if ((selectedIndex == -1) || (selectedIndex != tp.getSelectedIndex())) {
                        tp.setEnabledAt(tp.getSelectedIndex(), false);

                        // If this is not the first tab selection, then set the previously
                        // selected tab to "true".
                        if (selectedIndex != -1)
                                tp.setEnabledAt(selectedIndex, true);
                }
                // Store the index of the newly selected tab.
                selectedIndex = tp.getSelectedIndex();
        }

	/**
	********************************************************************************
	*	This method sets selected configuration to apply.
It also sets all
	*	necessary textfields.
	*	
	*	@param selected_config  The JTabbedPane index of the configuration to
	*				apply. 
	*
	*	@throws NumberFormatException If any textfield cannot be parsed.
	*	@throws Exception If any textfield contains invalid data.
	*	
	*	@see JTabbedPane
	*	@version 1.00
        *       @author Scott Streit
        ********************************************************************************
        */
        private void set_selected_configuration(int selected) throws NumberFormatException, Exception
	{
		if (selected == parent.controller_config_tabbed_pane.indexOfTab("Binning"))
			controller_config_cmd.select_binning(true);

		else if (selected == parent.controller_config_tabbed_pane.indexOfTab("Readout"))
			controller_config_cmd.select_readout(true);

		else if (selected == parent.controller_config_tabbed_pane.indexOfTab("MPP"))
			controller_config_cmd.select_mpp(true);

		else if (selected == parent.controller_config_tabbed_pane.indexOfTab("Coadder")) {
			controller_config_cmd.select_coadder(true);
			controller_config_cmd.set_number_of_coadds(parent.nc_txtfld.getText());
		}

		else if (selected == parent.controller_config_tabbed_pane.indexOfTab("Gain"))
			controller_config_cmd.select_gain(true);

		else if (selected == parent.controller_config_tabbed_pane.indexOfTab("Video Offsets")) {
			controller_config_cmd.select_video_offsets(true);
			for (int i = 0; i < 32; i++)
{
				controller_config_cmd.set_video_offsets(i, parent.channel_txtfld[i].getText());
			}
		}

		else if (selected == parent.controller_config_tabbed_pane.indexOfTab("Temperature")) {
			controller_config_cmd.select_temperature(true);
			controller_config_cmd.set_temperature(parent.temperature_txtfld.getText());
			ArrayTemperature.set_linear_coefficients(parent.linear_coeff1_txtfld.getText(), parent.linear_coeff2_txtfld.getText());
		}
	}
}
