/**
********************************************************************************
*	The class is responsible for taking an image exposure, reading out the
*	image, displaying the image, saving the image, and checking the image
*	data if the it is a DSP synthetically generated. This class is executed
*	inside of a SwingWorker thread class.
*
*	CVM - Model
*
*	@see ExposureCommand
*	@see SwingWorker
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import javax.swing.*;
import java.io.*;

public class DisplayZombie extends SwingWorker
{
	/**
	*****************************************************************************
	*	Constructor.
	*
	*	@see VoodooMainWindow
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public DisplayZombie()
	{
		// Superclass is automatically called, which starts the SwingWorker thread.
		
		// DO NOT initialize non-static members here when using SwingWorker.
		// Otherwise, accessing a member/method before the object becomes
		// instaniated in the JVM will cause a NullPointerException to
		// occur ... often.
	}

	/**
	*****************************************************************************
	*	This method starts the SwingWorker thread.
	*
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public Object construct()
	{
		return expose();
	}

	/**
	*****************************************************************************
	*	This method closes the SwingWorker thread. This method overides the
	*	one in Swingworker.java.
	*
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void finished() {}

	/**
	*****************************************************************************
	*	This method performs the exposure sequence.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public Object expose()
	{
		int rows = SetupCommand.get_ccd_rows();
		int cols = SetupCommand.get_ccd_cols();

		for (int x=0; x<20; x++) {
			try {
				Thread.sleep(100);
//				PrintMessage.message("ExposureCommand", 0, "Displaying image...waiting.\n", 0, true);
				ImageDisplay.Cdisplay(MemorySetup.mem_fd, rows, cols);
//				PrintMessage.message("ExposureCommand", 0, "done.", 8, false);
    			}
			catch (InterruptedException ie) {}
    			catch (Exception e) {
				PrintMessage.message("ExposureCommand", 1, "failed.", 8, false);
				JOptionPane.showMessageDialog(null, e.getMessage(), "Display Error", JOptionPane.ERROR_MESSAGE);
       			}
		}

		return "Done";
	}
}
