/**
********************************************************************************
*	A class to display a window from which the user may send
*	commands and receive replies from the contoller DSP's.
*
*	CVM - View
*
*	@see JDialog
*	@see ChangeListener
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

class DspDialog extends JDialog
{
	private Container content_pane;
	private JTabbedPane dsp_tabbed_pane;
	private	DspDialogListener dspDialog_listener = null;
	private JButton timing_send_cmd_button;
	private JButton manual_send_cmd_button;
	private JRadioButton timing_tdl_rbutton;
      	private JRadioButton timing_rdm_rbutton;
      	private JRadioButton timing_wrm_rbutton;
      	private Font font;
	private VTextField timing_reply_txtfld;
	private VTextField manual_reply_txtfld;
	private VTextField manual_command_txtfld;
	private VTextField manual_arg1_txtfld;
	private VTextField manual_arg2_txtfld;
	private VTextField manual_arg3_txtfld;
	private VTextField manual_arg4_txtfld;
	private VTextField manual_arg5_txtfld;
	protected VTextField timing_data_txtfld;
	protected VTextField timing_address_txtfld;
	public int number_of_dsp_windows;

	/**
	*****************************************************************************
	*	Constructor. All of the window components are created
	*	here.
	*
	*	@param owner The parent window to which this child window
	*		     belongs.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public DspDialog(Frame owner)
	{
		// Call the superclass (JDialog) constructor.
		super(owner, "DSP Command", false);
		
		// Set the dialog box size.
		setSize(340, 450);
		
		// Make the window non-resizable.
      		setResizable(false);
		
		// Get a handle on the dialog box's content pane.
		content_pane = this.getContentPane();

		// Set the general event listener.
		dspDialog_listener = new DspDialogListener(this);

		// Set the components font.
		font = new Font("LucidaSansRegular", Font.PLAIN, 11);

 		// Create and add the menu bar.
		JMenuBar new_menu_bar = create_menu_bar();
		setJMenuBar(new_menu_bar);

 		// Create and add the tool bar.
//		JToolBar new_tool_bar = create_toolbar();
//		content_pane.add(new_tool_bar, BorderLayout.NORTH);

		// Create the tabbed pane.
		JTabbedPane tp = create_tabbed_pane();
		content_pane.add(tp);
		
      		// Add a window listener.  This has to be here to allow the
      		// dialog box to be re-opened later when the "close" window
      		// menu item (from system, not java) is chosen.
      		addWindowListener(new WindowAdapter()
      		{
      			public void windowClosing(WindowEvent e)
      			{
      				number_of_dsp_windows = 0;
			}
		});
		
		// Initialize any variables.
		number_of_dsp_windows = 0;
	}

	/**
	*****************************************************************************
	*	Creates the "tabbed" panels portion of the window.
	*
	*	@return The created "tabbed pane" is returned.
	*
	*	@see JTabbedPane
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	private JTabbedPane create_tabbed_pane()
	{
		// Create a new tabbed pane and set the tabs to be on top.
		dsp_tabbed_pane = new JTabbedPane(JTabbedPane.TOP);
		
		// Set the tab event listener.
      		dsp_tabbed_pane.addChangeListener(dspDialog_listener);
      		
      		Component timing_panel = create_timing_panel();
      		dsp_tabbed_pane.addTab("PCI-TIM-UTIL", timing_panel);

      		Component manual_panel = create_manual_panel();
      		dsp_tabbed_pane.addTab("MANUAL", manual_panel);
      		
      		return dsp_tabbed_pane;
      	}

	/**
	*****************************************************************************
	*	Creates the PCI/Timing/Utility DSP command panel.
	*
	*	@return The created PCI/Timing/Utility DSP command
	*		panel is returned.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	private JPanel create_timing_panel()
	{
		JPanel timing_panel = new JPanel();
		
		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		timing_panel.setLayout(gbl);
      		
      		// Create the component event listener.
      		PciTimUtilCmdListener PciTimUtilCmd_listener = new PciTimUtilCmdListener(this);
      		
		// Initialize the command radio buttons.
      		timing_tdl_rbutton = new JRadioButton("TDL (Test Data Link)", false);
      		timing_rdm_rbutton = new JRadioButton("RDM (Read Memory)", false);
      		timing_wrm_rbutton = new JRadioButton("WRM (Write Memory)", false);

      		// Add the action listeners to the timing board buttons.
      		timing_tdl_rbutton.addActionListener(PciTimUtilCmd_listener);
      		timing_rdm_rbutton.addActionListener(PciTimUtilCmd_listener);
      		timing_wrm_rbutton.addActionListener(PciTimUtilCmd_listener);
      		
      		// Group the command radio buttons together so only one at a time may be selected.
      		ButtonGroup timing_rbutton_group = new ButtonGroup();
      		timing_rbutton_group.add(timing_tdl_rbutton);
      		timing_rbutton_group.add(timing_rdm_rbutton);
      		timing_rbutton_group.add(timing_wrm_rbutton);
      		
      		// Add the command radio buttons to the timing tabbed panel.
      		add_components(timing_panel, timing_tdl_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 0, 6, 1);
		add_components(timing_panel, timing_rdm_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 1, 6, 1);
      		add_components(timing_panel, timing_wrm_rbutton, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 2, 6, 1);

		// Create the board destination label.
		JLabel board_dest_label = new JLabel("Board: ");
		
		// Initialize the board destination radio buttons.
		JRadioButton pci_dest_rbutton = new JRadioButton("PCI", false);
      		JRadioButton timing_dest_rbutton = new JRadioButton("TIM", false);
      		JRadioButton utility_dest_rbutton = new JRadioButton("UTIL", false);

      		// Group the board destination radio buttons together so only one at a time may be selected.
      		ButtonGroup board_dest_rbutton_group = new ButtonGroup();
      		board_dest_rbutton_group.add(pci_dest_rbutton);
      		board_dest_rbutton_group.add(timing_dest_rbutton);
      		board_dest_rbutton_group.add(utility_dest_rbutton);

      		// Add the action listeners to the board destination buttons.
      		pci_dest_rbutton.addActionListener(PciTimUtilCmd_listener);
      		timing_dest_rbutton.addActionListener(PciTimUtilCmd_listener);
      		utility_dest_rbutton.addActionListener(PciTimUtilCmd_listener);

		// Set the default board destination. NOTE: This MUST be
		// done AFTER the listener is assigned.
		pci_dest_rbutton.doClick();

      		// Add the board destination radio buttons to the timing tabbed panel.
      		add_components(timing_panel, board_dest_label, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 3, 1, 1);
      		add_components(timing_panel, pci_dest_rbutton, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 3, 2, 1);
		add_components(timing_panel, timing_dest_rbutton, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 3, 2, 1);
      		add_components(timing_panel, utility_dest_rbutton, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 3, 2, 1);

		// Initialize the type timing radio buttons.
		JLabel timing_type_label = new JLabel("Type: ");
      		JRadioButton timing_R_rbutton = new JRadioButton("R", false);
      		JRadioButton timing_P_rbutton = new JRadioButton("P", false);
      		JRadioButton timing_X_rbutton = new JRadioButton("X", false);
      		JRadioButton timing_Y_rbutton = new JRadioButton("Y", false);

      		// Add the action listeners to the type buttons.
      		timing_R_rbutton.addActionListener(PciTimUtilCmd_listener);
      		timing_P_rbutton.addActionListener(PciTimUtilCmd_listener);
      		timing_X_rbutton.addActionListener(PciTimUtilCmd_listener);
      		timing_Y_rbutton.addActionListener(PciTimUtilCmd_listener);
      		
      		// Group the type radio buttons together so only one at a time may be selected.
      		ButtonGroup timing_type_rbutton_group = new ButtonGroup();
      		timing_type_rbutton_group.add(timing_R_rbutton);
      		timing_type_rbutton_group.add(timing_P_rbutton);
      		timing_type_rbutton_group.add(timing_X_rbutton);
      		timing_type_rbutton_group.add(timing_Y_rbutton);
      		
      		// Add the type radio buttons to the tabbed panel.
		add_components(timing_panel, timing_type_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 4, 1, 1);
		add_components(timing_panel, timing_R_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 4, 1, 1);
      		add_components(timing_panel, timing_P_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 4, 1, 1);
		add_components(timing_panel, timing_X_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 4, 1, 1);
      		add_components(timing_panel, timing_Y_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 4, 1, 1);

		// Initialize the timing board input text field labels.
		JLabel timing_address_label = new JLabel("Address:  0x");
		JLabel timing_data_label = new JLabel("Data:        0x");
		JLabel timing_reply_label = new JLabel("Reply:       0x");

		// Set the label color.
		board_dest_label.setForeground(Color.black);
		timing_type_label.setForeground(Color.black);
		timing_address_label.setForeground(Color.black);
		timing_data_label.setForeground(Color.black);
		timing_reply_label.setForeground(Color.black);
				
		// Initialize the timing board input text fields.
		timing_address_txtfld = new VTextField("",10);
		timing_data_txtfld = new VTextField("",10);
		timing_reply_txtfld = new VTextField("",10);

		// Set the reply textfield to uneditable.
		timing_reply_txtfld.setEditable(false);

		// Create the buttons.
      		timing_send_cmd_button = new JButton("Send Command");
		timing_send_cmd_button.setPreferredSize(new Dimension(120, 26));
		
		// Add the components.
		add_components(timing_panel, timing_address_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 5, 1, 1);
		add_components(timing_panel, timing_address_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 5, 5, 1);
		add_components(timing_panel, timing_data_label, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 6, 1, 1);
		add_components(timing_panel, timing_data_txtfld, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 6, 5, 1);
		add_components(timing_panel, timing_send_cmd_button, gbl, gbc, 0, 0, 20, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 11, 7, 1);
		add_components(timing_panel, timing_reply_label, gbl, gbc, 10, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 14, 1, 1);
		add_components(timing_panel, timing_reply_txtfld, gbl, gbc, 10, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 14, 5, 1);

      		
      		// Add the button lisenters.
		timing_send_cmd_button.addActionListener(PciTimUtilCmd_listener);
		
		return timing_panel;
	}

	/**
	*****************************************************************************
	*	Creates the Manual DSP command panel.
	*
	*	@return The created Manual DSP command panel is returned.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	private JPanel create_manual_panel()
	{
		JPanel manual_panel = new JPanel();
		
		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		manual_panel.setLayout(gbl);
      		
      		// Create the component event listener.
      		ManualCmdListener ManualCmd_listener = new ManualCmdListener(this);
      		
		// Initialize the board destination radio buttons.
		JRadioButton pci_dest_rbutton = new JRadioButton("PCI", true);
      		JRadioButton timing_dest_rbutton = new JRadioButton("TIM", false);
      		JRadioButton utility_dest_rbutton = new JRadioButton("UTIL", false);
      		
      		// Group the board destination radio buttons together so only one at a time may be selected.
      		ButtonGroup board_dest_rbutton_group = new ButtonGroup();
      		board_dest_rbutton_group.add(pci_dest_rbutton);
      		board_dest_rbutton_group.add(timing_dest_rbutton);
      		board_dest_rbutton_group.add(utility_dest_rbutton);

		// Initialize the labels.
		JLabel board_dest_label = new JLabel("Board: ");
		JLabel manual_command_label = new JLabel("Command:    ");
		JLabel manual_arg1_label = new JLabel("Argument 1: ");
		JLabel manual_arg2_label = new JLabel("Argument 2: ");
		JLabel manual_arg3_label = new JLabel("Argument 3: ");
		JLabel manual_arg4_label = new JLabel("Argument 4: ");
		JLabel manual_arg5_label = new JLabel("Argument 5: ");
		JLabel manual_reply_label = new JLabel("Reply:           0x");

		// Set the label color.
		board_dest_label.setForeground(Color.black);
		manual_command_label.setForeground(Color.black);
		manual_arg1_label.setForeground(Color.black);
		manual_arg2_label.setForeground(Color.black);
		manual_arg3_label.setForeground(Color.black);
		manual_arg4_label.setForeground(Color.black);
		manual_arg5_label.setForeground(Color.black);
		manual_reply_label.setForeground(Color.black);
		
		// Initialize the text fields.
		manual_command_txtfld = new VTextField("",10);
		manual_arg1_txtfld = new VTextField("",10);
		manual_arg2_txtfld = new VTextField("",10);
		manual_arg3_txtfld = new VTextField("",10);
		manual_arg4_txtfld = new VTextField("",10);
		manual_arg5_txtfld = new VTextField("",10);
		manual_reply_txtfld = new VTextField("",10);

		// Set the reply textfield to uneditable.
		manual_reply_txtfld.setEditable(false);
		
		// Create the button.
		manual_send_cmd_button = new JButton("Send Command");
		manual_send_cmd_button.setPreferredSize(new Dimension(120, 26));
		
		// Add the components to the panel.
      		add_components(manual_panel, board_dest_label, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 1, 1);
      		add_components(manual_panel, pci_dest_rbutton, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 0, 1, 1);
		add_components(manual_panel, timing_dest_rbutton, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 0, 1, 1);
      		add_components(manual_panel, utility_dest_rbutton, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 0, 1, 1);
		add_components(manual_panel, manual_command_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 2, 1, 1);
		add_components(manual_panel, manual_command_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 2, 6, 1);
		add_components(manual_panel, manual_arg1_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 3, 1, 1);
		add_components(manual_panel, manual_arg1_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 3, 6, 1);
		add_components(manual_panel, manual_arg2_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 4, 1, 1);
		add_components(manual_panel, manual_arg2_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 4, 6, 1);
		add_components(manual_panel, manual_arg3_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 5, 1, 1);
		add_components(manual_panel, manual_arg3_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 5, 6, 1);
		add_components(manual_panel, manual_arg4_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 6, 1, 1);
		add_components(manual_panel, manual_arg4_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 6, 6, 1);
		add_components(manual_panel, manual_arg5_label, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 7, 1, 1);
		add_components(manual_panel, manual_arg5_txtfld, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 7, 6, 1);
		add_components(manual_panel, manual_send_cmd_button, gbl, gbc, 20, 0, 20, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 10, 7, 1);
		add_components(manual_panel, manual_reply_label, gbl, gbc, 10, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 12, 1, 1);
		add_components(manual_panel, manual_reply_txtfld, gbl, gbc, 10, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 12, 6, 1);

		// Add the event listeners to the components      		
      		pci_dest_rbutton.addActionListener(ManualCmd_listener);
      		timing_dest_rbutton.addActionListener(ManualCmd_listener);
      		utility_dest_rbutton.addActionListener(ManualCmd_listener);
		manual_send_cmd_button.addActionListener(ManualCmd_listener);

      		// Set the default board destination.
		pci_dest_rbutton.doClick();

		return manual_panel;
	}
			   	
	/**
	*****************************************************************************
	*	Adds the specified component to the specified panel.
	*
	*	@param panel The panel to which the component will be added.
	*	@param compnt The component to add.
	*	@param gbl The GridBagLayout of the panel.
	*	@param gbl The GridBagConstraints of the panel.
	*	@param insetTOP The distance in pixels between the top of this
	*			component and the one above it.
	*	@param insetBOTTOM The distance in pixels between the bottom of
	*			this component and the one below it.
	*	@param insetLEFT The distance in pixels between the left side of
	*			this component and the one left of it.
	*	@param insetRIGHT The distance in pixels between the right side
	*			of this component and the one right of it.
	*	@param cfill Determines how the component "fills" the space of
	*		  	of its "cell".
	*	@param anchr The justification of the component within its "cell".
	*	@param col The x position of the component.
	*	@param row The y position of the component.
	*	@param gw The number of "cells" that the component spans in the
	*			width direction.
	*	@param gh The number of "cells" that the component spans in the
	*			height direction.
	*
	*	@see JPanel
	*	@see JComponent
	*	@see GridBagLayout
	*	@see GridBagConstraints
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
   	private void add_components(JPanel panel, JComponent compnt, GridBagLayout gbl, GridBagConstraints gbc, int insetTOP, int insetLEFT, int insetBOTTOM, int insetRIGHT, int cfill, int anchr, int col, int row, int gw, int gh)
   	{
      		// Set the grid layout constraints.
      		gbc.fill = cfill;
      		gbc.anchor = anchr;
      		gbc.insets = new Insets(insetTOP, insetLEFT, insetBOTTOM, insetRIGHT);
      		gbc.gridx = col;
      		gbc.gridy = row;
      		gbc.gridwidth = gw;
      		gbc.gridheight = gh;
      		
      		// Set the component font.
      		compnt.setFont(font);
      
      		// Add the component to the panel.
      		gbl.setConstraints(compnt, gbc);
      		panel.add(compnt);
      	}

	/**
	*****************************************************************************
	*	Sets the text for the pci-tim-util reply text field.
	*
	*	@param reply_string The text.
	*
	*	@see VTextField
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void set_timing_reply_txtfld(String reply_string)
	{
		timing_reply_txtfld.setText(reply_string.toUpperCase());
	}

	/**
	*****************************************************************************
	*	Sets the text for the manual reply text field.
	*
	*	@param reply_string The text.
	*
	*	@see VTextField
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void set_manual_reply_txtfld(String reply_string)
	{
		manual_reply_txtfld.setText(reply_string.toUpperCase());
	}

	/**
	********************************************************************************
	*	This method sets the send command button text. Used to change the
	*	button to an abort button while sending commands.
	*
	*	@param text The text to display on the expose button.
	*	@param window The window which contains the command button. May be one
	*		      of the following: "TimUtilPci", "Manual".
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
      	protected void set_command_button_text(String text, String window)
      	{
      		if (window.equals("TimUtilPci"))
      			timing_send_cmd_button.setText(text);
      		else if (window.equals("Manual"))
      			manual_send_cmd_button.setText(text);
      	}

	/**
	*****************************************************************************
	*	Retrieves the text in the command textfield of the Manual
	*	DSP command panel.
	*
	*	@return The String representation of the command is returned.
	*
	*	@see VTextField
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public String get_manual_command_txtfld()
	{
		return manual_command_txtfld.getText();
	}

	/**
	*****************************************************************************
	*	Retrieves the specified argument textfield of the Manual
	*	DSP command panel.
	*
	*	@return The String representation of the argument is returned.
	*
	*	@see VTextField
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public String get_manual_args_txtfld(int arg)
	{
		String txtfld_string = "";
		
		// Read the text from the appropriate argument textfield.
		switch (arg) {
			case 1:
				txtfld_string = manual_arg1_txtfld.getText();
				break;
			case 2: 
				txtfld_string = manual_arg2_txtfld.getText();
				break;
			case 3:
				txtfld_string = manual_arg3_txtfld.getText();
				break;
			case 4:
				txtfld_string = manual_arg4_txtfld.getText();
				break;
			case 5:
				txtfld_string = manual_arg5_txtfld.getText();
				break;
		}
		
		return txtfld_string;
	}

	/**
	*****************************************************************************
	*	Creates the menu bar for the DSP command window.
	*
	*	@return Returns the created menu bar.
	*
	*	@see JMenuBar
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	private JMenuBar create_menu_bar()
	{
		JMenuBar menu_bar = new JMenuBar();
		
		// Set the menubar's border.
		menu_bar.setBorder(new BevelBorder(BevelBorder.RAISED));
		menu_bar.setBorderPainted(true);
		
		// Add the Window menu to the menu bar.
		JMenu window_menu = new JMenu("Window");
		menu_bar.add(window_menu);
		
		// Add the submenus to the Window menu.
		JMenuItem close_item = new JMenuItem("Close", 'C');
		window_menu.add(close_item);
		
		// Set the menubar font.
		window_menu.setFont(font);
		close_item.setFont(font);

		// Add the event listeners to the menu items.
		close_item.addActionListener(dspDialog_listener);
		
		return menu_bar;
	}
	
	/**
	*****************************************************************************
	*	Creates the tool bar for the DSP command window.
	*
	*	@return Returns the created tool bar.
	*
	*	@see JToolBar
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	private JToolBar create_toolbar()
	{
		JToolBar toolbar = new JToolBar();
		
		// Set the margin between toolbar border and its components.
		toolbar.setMargin(new Insets(5, 5, 5, 5));
		
		// Set the toolbar to unfloatable.
		toolbar.setFloatable(true);
		
		toolbar.setBackground(Color.orange);
		
		// Create the toolbar buttons.
		JButton close_button = new JButton("Close");
		JButton help_button = new JButton("Help");
		
		// Add the buttons to the toolbar.
		toolbar.add(close_button);
		toolbar.addSeparator();
		toolbar.add(help_button);
		toolbar.addSeparator();
	
		return toolbar;
	}
}
