/**
********************************************************************************
*	The event handler class for the DSP command window.
*
*
*	CVM - Controller
*
*	@see DspDialog
*	@see ActionListener
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;

public class DspDialogListener implements ActionListener, ChangeListener
{
	private DspDialog parent = null;
	private int selectedIndex;
	private String message;
	
	/**
	*****************************************************************************
	*	Constructor.
	*
	*	@param parent The window for which this class listens.
	*
	*	@see DspDialog
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public DspDialogListener(DspDialog parent)
	{
		this.parent = parent;
		selectedIndex = -1;
	}

	/**
	*****************************************************************************
	*	This method overides the same one in the ActionListener
	*	interface. All events for the parent window are handled
	*	here.
	*
	*	@param e The event description.
	*
	*	@see ActionListener
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void actionPerformed(ActionEvent e)
	{
		AbstractButton active_button = (AbstractButton) e.getSource();
		
		// Close the Dsp Command window.
		if (active_button.getText().equals("Close")) {
			parent.setVisible(false);
			parent.number_of_dsp_windows = 0;
		}
	}

	/**
	*****************************************************************************
	*	Controls the switching of the "tabbed panes". This method
	*	overides the same method in the ChangeListener interface.
	*
	*	@see ChangeListener
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void stateChanged(ChangeEvent ce)
   	{
      		JTabbedPane tp = (JTabbedPane)ce.getSource();
       
      		// Check if a tab has been selected for the first time or if a new tab
      		// has been selected and set the tab to "false".
      		if ((selectedIndex == -1) || (selectedIndex != tp.getSelectedIndex())) {
      			tp.setEnabledAt(tp.getSelectedIndex(), false);
         	
         		// If this is not the first tab selection, then set the previously
         		// selected tab to "true".
         		if (selectedIndex != -1)
         			tp.setEnabledAt(selectedIndex, true);
      		}
      		// Store the index of the newly selected tab.
      		selectedIndex = tp.getSelectedIndex();
   	}
}