/**
********************************************************************************
*	A class to load or save CCD images in the FITS data type.
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.io.*;
import java.util.*;
import java.awt.image.*;

public class Fits
{
  	private FitsDialog parent = null;
  	
  	// C function prototypes.
  	public native void Cwrite_fits_data(String filename, int mem_fd, int rows, int cols, String header[], int bytes);

  	// Load the library that contains the C functions.
  	static
  	{
    		System.loadLibrary("fitslibc");
  	}

	/**
	********************************************************************************
	*	Default constructor. 
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public Fits(FitsDialog parent)
	{
		this.parent = parent;
	}

	/**
	********************************************************************************
	*	Writes the image <I>bytes</I> specified by the file descriptor
	*	to the file specified by <I>filename</I>.
	*
	*	@param fits_filename The name of the FITS file to be created.
	*	@param data_fd The raw image data file descriptor.
	*	@param bytes The number of image data bytes.
	*	@throws FileNotFoundException System could not create the file.
	*	@throws IOException System could not write to the file.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void write_fits_file(String fits_filename, int data_fd, int rows, int cols, int bytes) throws FileNotFoundException, IOException
	{
		// Retrieve the FITS header.
		String header[] = parent.get_header_info();

		Cwrite_fits_data(fits_filename, data_fd, rows, cols, header, bytes);
	}
}
