/**
********************************************************************************
*	A class to display a window from which the user may load,
*	edit and save FITS header information.
*
*	CVM - View
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.Calendar;
import java.util.GregorianCalendar;
import javax.swing.border.*;

class FitsDialog extends JDialog
{
	private Container content_pane;
	private JPanel new_fits_panel;
	private JMenuBar new_menu_bar;
	private VoodooMainWindow parent;
	private	VTextField simple_txtfld;
	private	VTextField bitpix_txtfld;
	private	VTextField naxis_txtfld;
	private	VTextField naxis1_txtfld;
	private	VTextField naxis2_txtfld;
	private	VTextField date_txtfld;
	private	VTextField time_txtfld;
	private	VTextField location_txtfld;
	private VTextField local_siderial_time_txtfld;
	private VTextField epoch_txtfld;
	private VTextField airmass_txtfld;
	private	VTextField exptime_txtfld;
	private	VTextField image_type_txtfld;
	private	VTextField telescope_txtfld;
	private	VTextField instrument_txtfld;
	private	VTextField filter_txtfld;
	private	VTextField object_txtfld;
	private	VTextField ra_txtfld;
	private	VTextField dec_txtfld;
	private	VTextField observer_txtfld;
	private	VTextField comment1_txtfld;
	private	VTextField comment2_txtfld;
	private	VTextField comment3_txtfld;
	private	VTextField comment4_txtfld;
	private FitsDialogListener fits_dialog_listener;
	private Font font;
	public int number_of_fits_windows;

	// Define constants.
	public static final int SIMPLE    = 0;
	public static final int BITPIX    = 1;
	public static final int NAXIS     = 2;
	public static final int NAXIS1    = 3;
	public static final int NAXIS2    = 4;
	public static final int DATE	  = 5;
	public static final int TIME      = 6;
	public static final int LOCATION  = 7;
	public static final int SIDETIME  = 8;
	public static final int EPOCH     = 9;
	public static final int AIRMASS   = 10;
	public static final int EXP_TIME  = 11;
	public static final int IMG_TYPE  = 12;
	public static final int TELESCOPE = 13;
	public static final int INSTRMENT = 14;
	public static final int FILTER    = 15;
	public static final int OBJECT    = 16;
	public static final int RA        = 17;
	public static final int DEC       = 18;
	public static final int OBSERVER  = 19;
	public static final int COMMENT1  = 20;
	public static final int COMMENT2  = 21;
	public static final int COMMENT3  = 22;
	public static final int COMMENT4  = 23;
	public static final int MAX_FITS_HEADER = 24;
	public static final int MAX_FITS_CARD_LENGTH = 20;
	
	/**
	********************************************************************************
	*	Class Constructor.
	*
	*	@param owner The parent window of this dialog box.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public FitsDialog(VoodooMainWindow owner)
	{
		// Call the superclass (JDialog) constructor.
		super(owner, "FITS File Control", false);
		
		// Save the windows parent.
		this.parent = owner;
		
		// Set the dialog box size.
		setSize(370, 640);
		
		// Make the window non-resizable.
      		setResizable(false);
		
		// Get a handle on the dialog box's content pane.
		content_pane = this.getContentPane();

		// Set the components font.
		font = new Font("LucidaSansRegular", Font.PLAIN, 11);

		// Set the event listener for all components.
		fits_dialog_listener = new FitsDialogListener(this);
		
		// Create the menu bar and add it to the dialog box.
		new_menu_bar = create_menu_bar();
		setJMenuBar(new_menu_bar);
		
		// Create the fits panel and add it to the dialog box.
		new_fits_panel = create_fits_panel();
		content_pane.add(new_fits_panel);
		
		// Display the dialog box.
		//this.show();

      		// Add a window listener.  This has to be here to allow the
      		// dialog box to be re-opened later when the "close" window
      		// menu item (from system, not java) is chosen.
      		addWindowListener(new WindowAdapter()
      		{
      			public void windowClosing(WindowEvent e)
      			{
      				number_of_fits_windows = 0;
			}
		});
				
		// Initialize any variables.
		number_of_fits_windows = 0;
	}

	/**
	********************************************************************************
	*	This method creates all the FITS interface components.
	*
	*	<PRE>
	*	<U>Card</U>		<U>Default Value</U>
	*	simple			"T"
	*	bitpix			"16"
	*	naxis			"2"
	*	naxis1			"0"
	*	naxis2			"0"
	*	date			"Date At Program Startup"
	*	time			"Time At Program Startup"
	*	location		"Observatory"
	*	Local Siderial Time	"hh:mm:ss.ss"
	*	Epoch			"2000"
	*	Airmass			"99.99"
	*	Exposure Time		"0.00"
	*	Image Type		"Image"
	*	Telescope		"Telescope"
	*	Instrument		"Instrument"
	*	Filter			"None"
	*	Object			"None"
	*	Right Ascension		"hh:mm:ss.ss"
	*	Declination		"hh:mm:ss.ss"
	*	Observer		"Observer"
	*	Comment1		"None"
	* 	Comment2		"None"
	*	Comment3		"None"
	*	Comment4		"None"
	*	</PRE>
	*
	*	@return Returns a JPanel that contains all the FITS input interface
	*		components.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public JPanel create_fits_panel()
	{
     		JPanel fits_panel = new JPanel();
     		
     		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		fits_panel.setLayout(gbl);

     		// Create the textfield labels.
     		JLabel simple_label = new JLabel("Simple:");
     		JLabel bitpix_label = new JLabel("Bitpix:");
     		JLabel naxis_label = new JLabel("Naxis:");
     		JLabel naxis1_label = new JLabel("Naxis1:");
      		JLabel naxis2_label = new JLabel("Naxis2:");
     		JLabel date_label = new JLabel("Date:");
     		JLabel time_label = new JLabel("Time:");
     		JLabel location_label = new JLabel("Location:");
     		JLabel local_siderial_time_label = new JLabel("Local Siderial Time:");
     		JLabel epoch_label = new JLabel("Epoch:");
   		JLabel airmass_label = new JLabel("Airmass:");
     		JLabel exptime_label = new JLabel("Exposure Time (sec):");
     		JLabel image_type_label = new JLabel("Image Type:");
     		JLabel telescope_label = new JLabel("Telescope:");
     		JLabel instrument_label = new JLabel("Instrument:");
     		JLabel filter_label = new JLabel("Filter:");
     		JLabel object_label = new JLabel("Object:");
     		JLabel ra_label = new JLabel("Right Ascension:");
     		JLabel dec_label = new JLabel("Declination:");
     		JLabel observer_label = new JLabel("Observer:");
     		JLabel comment1_label = new JLabel("Comment1:");
     		JLabel comment2_label = new JLabel("Comment2:");
     		JLabel comment3_label = new JLabel("Comment3:");
     		JLabel comment4_label = new JLabel("Comment4:");

		// Set the label color.
     		simple_label.setForeground(Color.black);
     		bitpix_label.setForeground(Color.black);
     		naxis_label.setForeground(Color.black);
     		naxis1_label.setForeground(Color.black);
      		naxis2_label.setForeground(Color.black);
     		date_label.setForeground(Color.black);
     		time_label.setForeground(Color.black);
     		location_label.setForeground(Color.black);
     		local_siderial_time_label.setForeground(Color.black);
     		epoch_label.setForeground(Color.black);
   		airmass_label.setForeground(Color.black);
     		exptime_label.setForeground(Color.black);
     		image_type_label.setForeground(Color.black);
     		telescope_label.setForeground(Color.black);
     		instrument_label.setForeground(Color.black);
     		filter_label.setForeground(Color.black);
     		object_label.setForeground(Color.black);
     		ra_label.setForeground(Color.black);
     		dec_label.setForeground(Color.black);
     		observer_label.setForeground(Color.black);
     		comment1_label.setForeground(Color.black);
     		comment2_label.setForeground(Color.black);
     		comment3_label.setForeground(Color.black);
     		comment4_label.setForeground(Color.black);
		
		// Create the textfields.
     		simple_txtfld = new VTextField("T", 16);
     		bitpix_txtfld = new VTextField("16", 16);
     		naxis_txtfld = new VTextField("2", 16);
     		naxis1_txtfld = new VTextField("0", 16);
     		naxis2_txtfld = new VTextField("0", 16);
     		date_txtfld = new VTextField(get_current_date(), 16);
     		time_txtfld = new VTextField(get_current_time(), 16);
     		location_txtfld = new VTextField("Observatory", 16);
		local_siderial_time_txtfld = new VTextField("hh:mm:ss.ss", 16);
		epoch_txtfld = new VTextField("2000", 16);
		airmass_txtfld = new VTextField("99.99", 16);
     		exptime_txtfld = new VTextField("0.00", 16);
		image_type_txtfld = new VTextField("Image", 16);
     		telescope_txtfld = new VTextField("Telescope", 16);
		instrument_txtfld = new VTextField("Instrument", 16);
		filter_txtfld = new VTextField("None", 16);
     		object_txtfld = new VTextField("None", 16);
     		ra_txtfld = new VTextField("hh:mm:ss.ss", 16);
     		dec_txtfld = new VTextField("hh:mm:ss.ss", 16);
 		observer_txtfld = new VTextField("Observer", 16);
  		comment1_txtfld = new VTextField("None", 16);
   		comment2_txtfld = new VTextField("None", 16);
    		comment3_txtfld = new VTextField("None", 16);
    		comment4_txtfld = new VTextField("None", 16);
     
		// Set the font, style and size for the textfields.
     		simple_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
     		bitpix_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
     		naxis_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
     		naxis1_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
     		naxis2_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
     		date_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		time_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		location_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		local_siderial_time_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		epoch_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		airmass_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		exptime_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		image_type_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		telescope_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		instrument_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14)); 
		filter_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14)); 
		object_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		ra_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
		dec_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
 		observer_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
  		comment1_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
   		comment2_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
    		comment3_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));
    		comment4_txtfld.setFont(new Font("SansSerif", Font.PLAIN, 14));

		// Set the fixed FITS cards to non-editable.
     		simple_txtfld.setEditable(false);
     		bitpix_txtfld.setEditable(false);
     		naxis_txtfld.setEditable(false);
     		naxis1_txtfld.setEditable(false);
     		naxis2_txtfld.setEditable(false);
				
		// Add the components to the panel.
      		add_components(fits_panel, simple_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 0, 1, 1);
      		add_components(fits_panel, simple_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 1, 0, 1, 1);
      		add_components(fits_panel, bitpix_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 1, 1, 1);
      		add_components(fits_panel, bitpix_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 1, 1, 1, 1);
      		add_components(fits_panel, naxis_label, gbl, gbc, 0, 0, 0, 5, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 2, 1, 1);
      		add_components(fits_panel, naxis_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 1, 2, 1, 1);
      		add_components(fits_panel, naxis1_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 3, 1, 1);
      		add_components(fits_panel, naxis1_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 1, 3, 1, 1);
      		add_components(fits_panel, naxis2_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 4, 1, 1);
      		add_components(fits_panel, naxis2_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 1, 4, 1, 1);
      		add_components(fits_panel, date_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 5, 1, 1);
      		add_components(fits_panel, date_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 5, 1, 1);
      		add_components(fits_panel, time_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 6, 1, 1);
      		add_components(fits_panel, time_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 6, 1, 1);
      		add_components(fits_panel, location_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 7, 1, 1);
      		add_components(fits_panel, location_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 7, 1, 1);
      		add_components(fits_panel, local_siderial_time_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 8, 1, 1);
      		add_components(fits_panel, local_siderial_time_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 8, 1, 1);
      		add_components(fits_panel, epoch_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 9, 1, 1);
      		add_components(fits_panel, epoch_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 9, 1, 1);
      		add_components(fits_panel, airmass_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 10, 1, 1);
      		add_components(fits_panel, airmass_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 10, 1, 1);
      		add_components(fits_panel, exptime_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 11, 1, 1);
      		add_components(fits_panel, exptime_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 11, 1, 1);
      		add_components(fits_panel, image_type_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 12, 1, 1);
      		add_components(fits_panel, image_type_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 12, 1, 1);
      		add_components(fits_panel, telescope_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 13, 1, 1);
      		add_components(fits_panel, telescope_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 13, 1, 1);
      		add_components(fits_panel, instrument_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 14, 1, 1);
      		add_components(fits_panel, instrument_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 14, 1, 1);
      		add_components(fits_panel, filter_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 15, 1, 1);
      		add_components(fits_panel, filter_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 15, 1, 1);
      		add_components(fits_panel, object_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 16, 1, 1);
      		add_components(fits_panel, object_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 16, 1, 1);
      		add_components(fits_panel, ra_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 17, 1, 1);
      		add_components(fits_panel, ra_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 17, 1, 1);
      		add_components(fits_panel, dec_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 18, 1, 1);
      		add_components(fits_panel, dec_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 18, 1, 1);
      		add_components(fits_panel, observer_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 19, 1, 1);
      		add_components(fits_panel, observer_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 19, 1, 1);
      		add_components(fits_panel, comment1_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 20, 1, 1);
      		add_components(fits_panel, comment1_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 20, 1, 1);
      		add_components(fits_panel, comment2_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 21, 1, 1);
      		add_components(fits_panel, comment2_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 21, 1, 1);
      		add_components(fits_panel, comment3_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 22, 1, 1);
      		add_components(fits_panel, comment3_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 22, 1, 1);
      		add_components(fits_panel, comment4_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 23, 1, 1);
      		add_components(fits_panel, comment4_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 23, 1, 1);

     		return fits_panel;
     	}

	/**
	********************************************************************************
	*	This method creates the menu bar for the FITS dialog box.
	*
	*	@return Returns a JMenuBar that contains all the FITS dialog box menu
	*		components.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JMenuBar create_menu_bar()
	{
		JMenuBar menu_bar = new JMenuBar();
		
		// Set the menubar's border.
		menu_bar.setBorder(new BevelBorder(BevelBorder.RAISED));
		menu_bar.setBorderPainted(true);
		
		// Add the Window menu to the menu bar.
		JMenu window_menu = new JMenu("Window");
		menu_bar.add(window_menu);
		
		// Add the submenus to the Window menu.
		JMenuItem close_item = new JMenuItem("Close", 'C');
		window_menu.add(close_item);

		// Set menubar font.
		window_menu.setFont(font);
		close_item.setFont(font);
				
		// Add the event listeners to the menu items.
		close_item.addActionListener(fits_dialog_listener);
		
		return menu_bar;
	}

	/**
	*****************************************************************************
	*	Adds the specified component to the specified panel.
	*
	*	@param panel The panel to which the component will be added.
	*	@param compnt The component to add.
	*	@param gbl The GridBagLayout of the panel.
	*	@param gbl The GridBagConstraints of the panel.
	*	@param insetTOP The distance in pixels between the top of this
	*			component and the one above it.
	*	@param insetBOTTOM The distance in pixels between the bottom of
	*			this component and the one below it.
	*	@param insetLEFT The distance in pixels between the left side of
	*			this component and the one left of it.
	*	@param insetRIGHT The distance in pixels between the right side
	*			of this component and the one right of it.
	*	@param cfill Determines how the component "fills" the space of
	*		  	of its "cell".
	*	@param anchr The justification of the component within its "cell".
	*	@param col The x position of the component.
	*	@param row The y position of the component.
	*	@param gw The number of "cells" that the component spans in the
	*			width direction.
	*	@param gh The number of "cells" that the component spans in the
	*			height direction.
	*
	*	@see JPanel
	*	@see JComponent
	*	@see GridBagLayout
	*	@see GridBagConstraints
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
    	private void add_components(JPanel panel, JComponent compnt, GridBagLayout gbl, GridBagConstraints gbc, int insetTOP, int insetLEFT, int insetBOTTOM, int insetRIGHT, int cfill, int anchr, int col, int row, int gw, int gh)
   	{
      		// Set the grid layout constraints.
      		gbc.fill = cfill;
      		gbc.anchor = anchr;
      		gbc.insets = new Insets(insetTOP, insetLEFT, insetBOTTOM, insetRIGHT);
      		gbc.gridx = col;
      		gbc.gridy = row;
      		gbc.gridwidth = gw;
      		gbc.gridheight = gh;
      		
      		// Set the component font.
      		compnt.setFont(font);
      
      		// Add the component to the panel.
      		gbl.setConstraints(compnt, gbc);
      		panel.add(compnt);
      	}


      	/**
	********************************************************************************
	*	This method returns the current date.
	*
	*	@return Returns a string containing the current date in the format MM/DD/YY.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private String get_current_date()
	{
     		GregorianCalendar calendar = new GregorianCalendar();
     		String month_string;
     		String day_string;
     		String year_string;
     		int month;
     		int day;
     		int year;
     		
     		//  Get the month, note that a 1 must be added because the
     		//  range of months returned by calender is 0-11.
     		if ((month = calendar.get(Calendar.MONTH) + 1) < 10)
     			month_string = "0" + Integer.toString(month);
		else
     			month_string = Integer.toString(month);
     			
     		// Get the day.
     		if ((day = calendar.get(Calendar.DATE)) < 10)
     			day_string = "0" + Integer.toString(day);
		else
     			day_string = Integer.toString(day);

     		// Get the year.
     		if ((year = calendar.get(Calendar.YEAR)) < 10)
     			year_string = "0" + Integer.toString(year);
		else
     			year_string = Integer.toString(year);

     		// Set the current date.
     		String date = month_string + "/" + day_string + "/" + year_string;

      		return date;
     	}

      	/**
	********************************************************************************
	*	This method returns the current time.
	*
	*	@return Returns a string containing the current time in the format HH:MM:SS.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private String get_current_time()
	{
     		GregorianCalendar calendar = new GregorianCalendar();
     		String hour_string;
     		String minutes_string;
     		String seconds_string;
     		int hour;
     		int minutes;
     		int seconds;
     		
     		// Get the hour.
     		if ((hour = calendar.get(Calendar.HOUR)) < 10)
     			hour_string = "0" + Integer.toString(hour);
		else
     			hour_string = Integer.toString(hour);
     			
     		// Get the minutes.
     		if ((minutes = calendar.get(Calendar.MINUTE)) < 10)
     			minutes_string = "0" + Integer.toString(minutes);
		else
     			minutes_string = Integer.toString(minutes);

     		// Get the seconds.
     		if ((seconds = calendar.get(Calendar.SECOND)) < 10)
     			seconds_string = "0" + Integer.toString(seconds);
		else
     			seconds_string = Integer.toString(seconds);

     		// Set the current time.
     		String time = hour_string + ":" + minutes_string + ":" + seconds_string;
     		       
     		return time;
     	}

      	/**
	********************************************************************************
	*	This method gets the FITS header textfield values.
	*
	*	@return Returns the current FITS header strings. The current date
	*		and time are taken at this time.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	protected String[] get_header_info()
	{
		String fits_header[] = new String[MAX_FITS_HEADER];
		
		fits_header[SIMPLE] = "T";
		fits_header[BITPIX] = Integer.toString(parent.setup_dialog.setup_listener.setup_cmd.get_image_resolution());
		fits_header[NAXIS] = "2";
		fits_header[NAXIS1] = Integer.toString(parent.setup_dialog.setup_listener.setup_cmd.get_ccd_cols());
		fits_header[NAXIS2] = Integer.toString(parent.setup_dialog.setup_listener.setup_cmd.get_ccd_rows());
		fits_header[DATE] = date_txtfld.getText();
		fits_header[TIME] = time_txtfld.getText();
		fits_header[LOCATION] = location_txtfld.getText();
		fits_header[SIDETIME] = local_siderial_time_txtfld.getText();
		fits_header[EPOCH] = epoch_txtfld.getText();
		fits_header[AIRMASS] = airmass_txtfld.getText();
		fits_header[EXP_TIME] = exptime_txtfld.getText();
		fits_header[IMG_TYPE] = image_type_txtfld.getText();
		fits_header[TELESCOPE] = telescope_txtfld.getText();
		fits_header[INSTRMENT] = instrument_txtfld.getText();
		fits_header[FILTER] = filter_txtfld.getText();
		fits_header[OBJECT] = object_txtfld.getText();
		fits_header[RA] = ra_txtfld.getText();
		fits_header[DEC] = dec_txtfld.getText();
		fits_header[OBSERVER] = observer_txtfld.getText();
		fits_header[COMMENT1] = comment1_txtfld.getText();
		fits_header[COMMENT2] = comment2_txtfld.getText();
		fits_header[COMMENT3] = comment3_txtfld.getText();
		fits_header[COMMENT4] = comment4_txtfld.getText();

		// Crop the header string to be no longer than MAX_FITS_CARD_LENGTH characters.
		for (int i=1; i<MAX_FITS_HEADER; i++) {
			if (fits_header[i].length() > MAX_FITS_CARD_LENGTH)
				fits_header[i] = fits_header[i].substring(0, MAX_FITS_CARD_LENGTH-1);
		}

		// Update the text fields with new "cropped" values.
		simple_txtfld.setText(fits_header[SIMPLE]);
		bitpix_txtfld.setText(fits_header[BITPIX]);
		naxis_txtfld.setText(fits_header[NAXIS]);
		naxis1_txtfld.setText(fits_header[NAXIS1]);
		naxis2_txtfld.setText(fits_header[NAXIS2]);
		date_txtfld.setText(fits_header[DATE]);
		time_txtfld.setText(fits_header[TIME]);
		location_txtfld.setText(fits_header[LOCATION]);
		local_siderial_time_txtfld.setText(fits_header[SIDETIME]);
		epoch_txtfld.setText(fits_header[EPOCH]);
		airmass_txtfld.setText(fits_header[AIRMASS]);
		exptime_txtfld.setText(fits_header[EXP_TIME]);
		image_type_txtfld.setText(fits_header[IMG_TYPE]);
		telescope_txtfld.setText(fits_header[TELESCOPE]);
		instrument_txtfld.setText(fits_header[INSTRMENT]);
		filter_txtfld.setText(fits_header[FILTER]);
		object_txtfld.setText(fits_header[OBJECT]);
		ra_txtfld.setText(fits_header[RA]);
		dec_txtfld.setText(fits_header[DEC]);
		observer_txtfld.setText(fits_header[OBSERVER]);
		comment1_txtfld.setText(fits_header[COMMENT1]);
		comment2_txtfld.setText(fits_header[COMMENT2]);
		comment3_txtfld.setText(fits_header[COMMENT3]);
		comment4_txtfld.setText(fits_header[COMMENT4]);

		return fits_header;
	}

      	/**
	********************************************************************************
	*	This method updates the FITS header Bitpix, Row, Column, Date, and Time
	*	textfields.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	protected void update_fits_info()
	{
		// Update the text fields with new "cropped" values.
		bitpix_txtfld.setText(Integer.toString(parent.setup_dialog.setup_listener.setup_cmd.get_image_resolution()));
		naxis1_txtfld.setText(Integer.toString(parent.setup_dialog.setup_listener.setup_cmd.get_ccd_cols()));
		naxis2_txtfld.setText(Integer.toString(parent.setup_dialog.setup_listener.setup_cmd.get_ccd_rows()));
		date_txtfld.setText(get_current_date());
		time_txtfld.setText(get_current_time());
		exptime_txtfld.setText(parent.exposure_time_txtfld.getText());
	}
}
