/**
********************************************************************************
*	A class to display a window from which the user may load,
*	edit and save FITS header information.
*
*	CVM - View
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

class HardwareTestDialog extends JDialog
{
	private SetupDialog parent;
	private Container content_pane;
	private JPanel new_test_panel;
	private int pci_test_num;
	private int timing_test_num;
	private int utility_test_num;
	private Font font;
	protected JTextField pci_test_txtfld;
	protected JTextField timing_test_txtfld;
	protected JTextField utility_test_txtfld;
     	protected JCheckBox pci_chkbox;
     	protected JCheckBox timing_chkbox;
     	protected JCheckBox utility_chkbox;
	protected int number_of_test_windows;
	
	/**
	********************************************************************************
	*	Class Constructor.
	*
	*	@param owner The parent window of this dialog box.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public HardwareTestDialog(SetupDialog owner)
	{
		// Call the superclass (JDialog) constructor.
		super(owner, "Hardware Test Options", false);
		
		// Save the parent;
		this.parent = owner;

		// Set the dialog box size.
		setSize(330, 180);
		
		// Make the window non-resizable.
      		setResizable(false);
		
		// Get a handle on the dialog box's content pane.
		content_pane = this.getContentPane();

		// Set the components font.
		font = new Font("LucidaSansRegular", Font.PLAIN, 11);
		
		// Initialize any variables.
		number_of_test_windows = 0;
		pci_test_num = 10;
		timing_test_num = 10;
		utility_test_num = 10;

		// Create the panel and add it to the dialog box.
		new_test_panel = create_test_panel();
		content_pane.add(new_test_panel);

      		// Add a window listener.  This has to be here to allow the
      		// dialog box to be re-opened later when the "close" window
      		// menu item (from system, not java) is chosen.
      		addWindowListener(new WindowAdapter()
      		{
      			public void windowClosing(WindowEvent e)
      			{
      				number_of_test_windows = 0;
			}
		});
	}

	/**
	********************************************************************************
	*	This method creates all the input interface components for the hardware
	*	test dialog box.
	*
	*	@return Returns a panel containing all the input interface components.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public JPanel create_test_panel()
	{
     		JPanel hardware_test_panel = new JPanel();
     		
    		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		hardware_test_panel.setLayout(gbl);

     		// Create the checkboxes.
     		pci_chkbox = new JCheckBox("PCI Board: ");
     		timing_chkbox = new JCheckBox("Timing Board: ");
     		utility_chkbox = new JCheckBox("Utility Board: ");

		// Set the checkbox action commands.
		pci_chkbox.setActionCommand("PCI Board Hardware Test");
		timing_chkbox.setActionCommand("Timing Board Hardware Test");
		utility_chkbox.setActionCommand("Utility Board Hardware Test");
		
     		// Create the labels.
     		JLabel tests1_label = new JLabel(" Tests");
     		JLabel tests2_label = new JLabel(" Tests");
		JLabel tests3_label = new JLabel(" Tests");
		
		// Set the label colors.
		tests1_label.setForeground(Color.black);
		tests2_label.setForeground(Color.black);
		tests3_label.setForeground(Color.black);
		
		// Create the textfields.
     		pci_test_txtfld = new JTextField(Integer.toString(pci_test_num), 10);
     		timing_test_txtfld = new JTextField(Integer.toString(timing_test_num), 10);
     		utility_test_txtfld = new JTextField(Integer.toString(utility_test_num), 10);
     		
   		// Create the buttons.
     		JButton close_button = new JButton("Close");

		// Set the button action commands.
		close_button.setActionCommand("Close Hardware Test");

		// Add the components to the panel.
      		add_components(hardware_test_panel, pci_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 0, 1, 1);
      		add_components(hardware_test_panel, pci_test_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 1, 0, 1, 1);
      		add_components(hardware_test_panel, tests1_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 2, 0, 1, 1);
      		add_components(hardware_test_panel, timing_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 1, 1, 1);
      		add_components(hardware_test_panel, timing_test_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 1, 1, 1, 1);
      		add_components(hardware_test_panel, tests2_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 2, 1, 1, 1);
      		add_components(hardware_test_panel, utility_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 2, 1, 1);
      		add_components(hardware_test_panel, utility_test_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 1, 2, 1, 1);
      		add_components(hardware_test_panel, tests3_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 2, 2, 1, 1);
      		add_components(hardware_test_panel, close_button, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 4, 3, 1);

		// Add the event listeners for the checkboxes.
		close_button.addActionListener(parent.setup_listener);
		pci_chkbox.addItemListener(parent.setup_listener);
		timing_chkbox.addItemListener(parent.setup_listener);
		utility_chkbox.addItemListener(parent.setup_listener);

		// Select the checkboxes.
		pci_chkbox.doClick();
		timing_chkbox.doClick();
		utility_chkbox.doClick();
		
     		return hardware_test_panel;
     	}

	/**
	*****************************************************************************
	*	Adds the specified component to the specified panel.
	*
	*	@param panel The panel to which the component will be added.
	*	@param compnt The component to add.
	*	@param gbl The GridBagLayout of the panel.
	*	@param gbl The GridBagConstraints of the panel.
	*	@param insetTOP The distance in pixels between the top of this
	*			component and the one above it.
	*	@param insetBOTTOM The distance in pixels between the bottom of
	*			this component and the one below it.
	*	@param insetLEFT The distance in pixels between the left side of
	*			this component and the one left of it.
	*	@param insetRIGHT The distance in pixels between the right side
	*			of this component and the one right of it.
	*	@param cfill Determines how the component "fills" the space of
	*		  	of its "cell".
	*	@param anchr The justification of the component within its "cell".
	*	@param col The x position of the component.
	*	@param row The y position of the component.
	*	@param gw The number of "cells" that the component spans in the
	*			width direction.
	*	@param gh The number of "cells" that the component spans in the
	*			height direction.
	*
	*	@see JPanel
	*	@see JComponent
	*	@see GridBagLayout
	*	@see GridBagConstraints
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
   	private void add_components(JPanel panel, JComponent compnt, GridBagLayout gbl, GridBagConstraints gbc, int insetTOP, int insetLEFT, int insetBOTTOM, int insetRIGHT, int cfill, int anchr, int col, int row, int gw, int gh)
   	{
      		// Set the grid layout constraints.
      		gbc.fill = cfill;
      		gbc.anchor = anchr;
      		gbc.insets = new Insets(insetTOP, insetLEFT, insetBOTTOM, insetRIGHT);
      		gbc.gridx = col;
      		gbc.gridy = row;
      		gbc.gridwidth = gw;
      		gbc.gridheight = gh;
      		
      		// Set the component font.
      		compnt.setFont(font);
      
      		// Add the component to the panel.
      		gbl.setConstraints(compnt, gbc);
      		panel.add(compnt);
      	}

	/**
	********************************************************************************
	*	This method returns the number (as a String) of PCI interface board
	*	tests to perform.
	*
	*	@return Returns the number of PCI interface board tests to perform.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
  	public String get_pci_test_number()
  	{
  		return pci_test_txtfld.getText();
  	}

	/**
	********************************************************************************
	*	This method returns the number (as a String) of Timing board tests
	*	to perform.
	*
	*	@return Returns the number of Timing board tests to perform.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public String get_timing_test_number()
  	{
  		return timing_test_txtfld.getText();
  	}

	/**
	********************************************************************************
	*	This method returns the number (as a String) of Utility board tests
	*	to perform.
	*
	*	@return Returns the number of Utility board tests to perform.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
  	public String get_utility_test_number()
  	{
  		return utility_test_txtfld.getText();
  	}
}
