/**
********************************************************************************
*	This class is the event listener for the manual commands input from the
*	DSP manual commands window.
*
*	CVM - Controller
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;

public class ManualCmdListener implements ActionListener
{
	private DspDialog parent = null;
	private ManualCommand manual_cmd = null;
	private int reply_value;
	
	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public ManualCmdListener(DspDialog parent)
	{
		this.parent = parent;
		manual_cmd = new ManualCommand(parent);
	}
		
	/**
	********************************************************************************
	*	This method handles all events from the DSP manual command window. All
	*	events are sent to the ManualCommand class.
	*
	*	@see ActionListener
	*	@see ManualCommand
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void actionPerformed(ActionEvent e)
	{
		AbstractButton active_button = (AbstractButton) e.getSource();
		reply_value = 0;
	
		if (active_button.getText().equals("PCI")) {
			manual_cmd.set_board_destination(VoodooConstants.PCI_ID);
		}
		
		else if (active_button.getText().equals("TIM")) {
			manual_cmd.set_board_destination(VoodooConstants.TIM_ID);
		}
		
		else if (active_button.getText().equals("UTIL")) {
			manual_cmd.set_board_destination(VoodooConstants.UTIL_ID);
		}

		else if (active_button.getText().equals("Send Command")) {
			// Clear the reply textfield.
			parent.set_manual_reply_txtfld("");
			
			// Get the command input textfield.
			try {
				manual_cmd.set_command(parent.get_manual_command_txtfld());
			}
			catch (ArrayIndexOutOfBoundsException aioobe) {
				PrintMessage.message("ManualCmdListener", 1, "ERROR: Command must be specified.\n", 0, true);
				return;
			}
			
			// Get the argument textfields.
			for (int i=1; i<=5; i++)
				manual_cmd.set_argument(i, parent.get_manual_args_txtfld(i));
			
			// Change the send command button text.
			parent.set_command_button_text("Abort Command", "Manual");
			
			// Send the command.
			manual_cmd.thread_it();
		}
		
		// Abort the current attempt to send a command.
		else if (active_button.getText().equals("Abort Command")) {
			manual_cmd.stop();
			manual_cmd.interrupt();
			System.gc();
			
			try {
				DspCommand2.flush_reply_buffer();
			}
			catch (Exception ex) {}
						
			// Set the apply button text to "Send Command".
			parent.set_command_button_text("Send Command", "Manual");
		}
	}
}