/**
********************************************************************************
*	This class creates and initializes a memory region of the specified size
*	for image storage.
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import javax.swing.JOptionPane;

public class MemorySetup
{
  	public static int mem_fd;
  	public static int map_fd;
  	
  	// C function prototypes.
  	public native static int Ccreate_memory(int buffer_size);
  	public native static int Cmap_memory(int pci_fd, int buffer_size);
  	public native static void Cunmap_memory(int map_fd, int buffer_size);
  	public native static int Cget_memory_word(int fd, int addr);
  	public native static void Cfree_memory(int fd);
  	public native static void Cswap_memory(int fd, int mem_length);
  	public native static int Cprint_memory(int fd, int bytes);
  	public native static int Cdata_check(int fd, int mem_word_length);
  	public native static int[] Craw2java(int fd, int[] pix, int row);

  	// Load the library that contains the C functions.
  	static
  	{
    		System.loadLibrary("pcimemc");
  	}

	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
  	public MemorySetup()
  	{
    		mem_fd = 0;
   		map_fd = 0;
  	}

	/**
	********************************************************************************
	*	This method creates a buffer for image storage.
	*
	*	@param buffer_size The size (in bytes) of the image buffer.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
  	public static void create_memory(int buffer_size)
  	{
    		try {
			mem_fd = Ccreate_memory(buffer_size);
	   	}
	   	catch (Exception e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Memory Allocation Error", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
	   	}
    	}

	/**
	********************************************************************************
	*	This class releases the allocated image buffer memory back to the system.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
    	public static void free_memory()
  	{
    		Cfree_memory(mem_fd);
    		mem_fd = 0;
    		System.out.println("\n Memory freed!");
    	}
}
