/**
********************************************************************************
*	This class is the event listener for the PCI-TIM-UTIL DSP command window.
*	All actions handled by this class are passed to the DSPCommand class.
*
*	CVM - Controller
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;
import java.sql.Types;

public class PciTimUtilCmdListener implements ActionListener
{
	private DspDialog parent = null;
	private PciTimUtilCommand PciTimUtil_cmd = null;
	private boolean do_tdl;
	private boolean do_rdm;
	private boolean do_wrm;
	private int reply_value;
	
	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@param parent The dialog window that this class listens for.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public PciTimUtilCmdListener(DspDialog parent)
	{
		this.parent = parent;
		PciTimUtil_cmd = new PciTimUtilCommand(parent);
	}
		
	/**
	********************************************************************************
	*	This method handles all the events.
	*
	*	@param e The event.
	*
	*	@see ActionListener
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void actionPerformed(ActionEvent e)
	{
		AbstractButton active_button = (AbstractButton) e.getSource();
		
		reply_value = 0;
	
		if (active_button.getText().equals("TDL (Test Data Link)")) {
			// Make sure the data input textfield is editable.
			parent.timing_data_txtfld.setEditable(true);

			// Set the command to TEST_DATA_LINK.
			PciTimUtil_cmd.set_command(VoodooConstants.TEST_DATA_LINK);
			do_tdl = true;
			do_rdm = false;
			do_wrm = false;
			
			// Make the address input textfield uneditable.
			parent.timing_address_txtfld.setEditable(false);
		}
		
		else if (active_button.getText().equals("RDM (Read Memory)")) {
			// Make sure the address input textfield is editable.
			parent.timing_address_txtfld.setEditable(true);
			
			// Set the command to READ_MEMORY.
			PciTimUtil_cmd.set_command(VoodooConstants.READ_MEMORY);
			do_tdl = false;
			do_rdm = true;
			do_wrm = false;
			
			// Set the data input text field to uneditable.
			parent.timing_data_txtfld.setEditable(false);
		}

		else if (active_button.getText().equals("WRM (Write Memory)")) {
			// Make sure the address and data input textfields are editable.
			parent.timing_address_txtfld.setEditable(true);
			parent.timing_data_txtfld.setEditable(true);
			do_tdl = false;
			do_rdm = false;
			do_wrm = true;
			
			// Set the command to WRITE_MEMORY.
			PciTimUtil_cmd.set_command(VoodooConstants.WRITE_MEMORY);
		}
		
		else if (active_button.getText().equals("PCI")) {
			PciTimUtil_cmd.set_board(VoodooConstants.PCI_ID);
		}
		
		else if (active_button.getText().equals("TIM")) {
			PciTimUtil_cmd.set_board(VoodooConstants.TIM_ID);
		}
		
		else if (active_button.getText().equals("UTIL")) {
			PciTimUtil_cmd.set_board(VoodooConstants.UTIL_ID);
		}
		
		else if (active_button.getText().equals("D")) {
			PciTimUtil_cmd.set_type(VoodooConstants.D);
		}
		
		else if (active_button.getText().equals("R")) {
			PciTimUtil_cmd.set_type(VoodooConstants.R);
		}
		
		else if (active_button.getText().equals("P")) {
			PciTimUtil_cmd.set_type(VoodooConstants.P);
		}
		
		else if (active_button.getText().equals("X")) {
			PciTimUtil_cmd.set_type(VoodooConstants.X);
		}
		
		else if (active_button.getText().equals("Y")) {
			PciTimUtil_cmd.set_type(VoodooConstants.Y);
		}
		
		else if (active_button.getText().equals("Send Command")) {
			// Clear the rely textfield.
			parent.set_timing_reply_txtfld("");
			
			// Validate the textfield data. If OK, send it.
			try {
				if (do_rdm || do_wrm) {
					PciTimUtil_cmd.set_address(parent.timing_address_txtfld.getText());
				}
				
				if (do_tdl || do_wrm) {
					PciTimUtil_cmd.set_data(parent.timing_data_txtfld.getText());
				}
			}
			catch (NumberFormatException nfe) {
				PrintMessage.message("PciTimUtilCmdListener", 1, nfe.getMessage() + ".\n", 0, true);
				return;
			}

			// Change the send command button text.
			parent.set_command_button_text("Abort Command", "TimUtilPci");
			
			// Send the command.
			PciTimUtil_cmd.thread_it();
		}
		
		// Abort the current attempt to send a command.
		else if (active_button.getText().equals("Abort Command")) {
			PciTimUtil_cmd.stop();
			PciTimUtil_cmd.interrupt();

			try {
				DspCommand2.flush_reply_buffer();
			}
			catch (Exception ex) {}

			// Set the apply button text to "Send Command".
			parent.set_command_button_text("Send Command", "TimUtilPci");
		}
	}
}