/**
********************************************************************************
*	A class that passes software and hardware error and  informational
*	messages to the user. The messages are	output to the info/status area
*	of the programs main application window.
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.Font;
import java.awt.Color;

public class PrintMessage
{
	private static int style = 1;
	
	/**
	********************************************************************************
	*	Default constructor.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public PrintMessage() {}
	
	/**
	********************************************************************************
	*	Passes the specifed message to the programs main
	*	application window for display.
	*
	*	@param message_string The message to display. The
	*			      text must have the following format:
	*
	*		<BR><PRE>
	*		<I>"<severity:> message"</I>
	*
	*		severity = ERROR or none.
	*		clazz    = The name of the class where the message is output.
	*		message  = The String to be output.
	*
	*		<U>For Example</U>:
	*		"ERROR: No file found."
	*		"Reading Image ..."
	*		</PRE>
	*
	*	@param offset The number of characters to replace when changing the
	*		      "...waiting." text to "...done.".
	*	@param start Specifies whether or not the message is the beginning or
	*		     end of a two part message. True = Start of message. False
	*		     = End of message. Single messages should use "true" and
	*		     end with a "\n".
	*
	*		     <br><pre>For example:
	*				"Doing readout...waiting." is a message beginning.
	*				Replacing "waiting." with "done." is a message end.
	*		     </pre>
	*
	*	@see VoodooMainWindow
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public static void message(String clazz, int severity, String message_string, int offset, boolean start)
	{
		VoodooMainWindow.increment_info_panel_scrollbar();

		if (severity == VoodooConstants.FAILURE)
			style = 2;
		else
			style = 1;


		if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON) {
			//VoodooMainWindow.set_info_panel_text("\n", style, offset);

			if (start == true)
				VoodooMainWindow.set_info_panel_text(message_string, style, offset);
			else {
				// This is to allow ERROR messages to be interleaved with DEBUG messages.
				if (severity == VoodooConstants.FAILURE)
					VoodooMainWindow.set_info_panel_text(style, message_string);
				else
					VoodooMainWindow.set_info_panel_text(style, message_string); //.substring(0, message_string.indexOf("\n")));
			}
		}
			
		else {
			if (start == true)
				VoodooMainWindow.set_info_panel_text(message_string, style, offset);
			else
				VoodooMainWindow.set_info_panel_text(style, message_string);
		}

		VoodooMainWindow.increment_info_panel_scrollbar();
	}

	/**
	********************************************************************************
	*	Prints additional information if the application debug mode is turned on.
	*
	*	@param type The message severity. May be <I>Info</I> or <I>Error</I>.
	*	@param source_class The class where the message originated.
	*	@param message_string The message.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
/*
	public static void debug_message(String message_type, String source_class, String message_string)
	{
		VoodooMainWindow.increment_info_panel_scrollbar();

		style = 3;
		VoodooMainWindow.set_info_panel_text(message_type + " (" + source_class + "): " + message_string + "\n", style);

		VoodooMainWindow.increment_info_panel_scrollbar();
	}
*/
	/**
	********************************************************************************
	*	Prints additional information if the application debug mode is turned on.
	*	Messages are printed using character style 3 as defined in
	*	<i>VoodooMainWindow</i>.
	*
	*	@param dest_arg The board destination and number of argument registers..
	*	@param cmd_description A string description of the command.
	*	@param cmd The command that produced the message.
	*	@param return_value The value returned as a result of using <I>cmd</I>.
	*
	*	@see VoodooMainWindow
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public static void debug_message(int dest_arg, String cmd_description, int cmd, int return_value)
	{
		VoodooMainWindow.increment_info_panel_scrollbar();

		style = 3;
		VoodooMainWindow.set_info_panel_text("Command Issued: " + cmd_description + " (0x" + Integer.toHexString(cmd) + ")\n", style);
		VoodooMainWindow.set_info_panel_text("Arg#/Destination: 0x" + Integer.toHexString(dest_arg) + "\n", style);
		VoodooMainWindow.set_info_panel_text("Return Value: 0x" + Integer.toHexString(return_value) + "\n", style);

		VoodooMainWindow.increment_info_panel_scrollbar();
	}

	/**
	********************************************************************************
	*	Prints additional information if the application debug mode is turned on.
	*	Messages are printed using character style 3 as defined in
	*	<i>VoodooMainWindow</i>.
	*
	*	@param dest_arg The board destination and number of argument registers..
	*	@param cmd_description A string description of the command.
	*	@param cmd The command that produced the message.
	*	@param arg1 The value of the argument 1 register.
	*	@param arg2 The value of the argument 2 register.
	*	@param arg3 The value of the argument 3 register.
	*	@param arg4 The value of the argument 4 register.
	*	@param arg5 The value of the argument 5 register.
	*	@param return_value The value returned as a result of using <I>cmd</I>.
	*
	*	@see VoodooMainWindow
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public static void debug_message(int dest_arg, String cmd_description, int cmd, int arg1, int arg2, int arg3, int arg4, int arg5, int return_value)
	{
		VoodooMainWindow.increment_info_panel_scrollbar();

		style = 3;
		VoodooMainWindow.set_info_panel_text("Command Issued: " + cmd_description + " (0x" + Integer.toHexString(cmd) + ")\n", style);
		VoodooMainWindow.set_info_panel_text("Arg#/Destination: 0x" + Integer.toHexString(dest_arg) + "\n", style);
		VoodooMainWindow.set_info_panel_text("Cmd Arg (1-5): 0x" + Integer.toHexString(arg1) + ", 0x" + Integer.toHexString(arg2) +
								      ", 0x" + Integer.toHexString(arg3) + ", 0x" + Integer.toHexString(arg4) +
								      ", 0x" + Integer.toHexString(arg5) + "\n", style);
		VoodooMainWindow.set_info_panel_text("Return Value: 0x" + Integer.toHexString(return_value) + "\n", style);

		VoodooMainWindow.increment_info_panel_scrollbar();
	}

	/**
	********************************************************************************
	*	Prints additional information if the application debug mode is turned on.
	*	This method does not contain a command description string, instead it
	*	attempts to convert the command to a three letter ascii string. Messages
	*	are printed using character style 3 as defined in <i>VoodooMainWindow</i>.
	*
	*	@param dest_arg The board destination and number of argument registers..
	*	@param cmd The command that produced the message.
	*	@param arg1 The value of the argument 1 register.
	*	@param arg2 The value of the argument 2 register.
	*	@param arg3 The value of the argument 3 register.
	*	@param arg4 The value of the argument 4 register.
	*	@param arg5 The value of the argument 5 register.
	*	@param return_value The value returned as a result of using <I>cmd</I>.
	*
	*	@see VoodooMainWindow
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public static void debug_message(int dest_arg, int cmd, int arg1, int arg2, int arg3, int arg4, int arg5, int return_value)
	{
		VoodooMainWindow.increment_info_panel_scrollbar();

		style = 3;
		VoodooMainWindow.set_info_panel_text("Command Issued: " + int_to_ascii(cmd) + " (0x" + Integer.toHexString(cmd) + ")\n", style);
		VoodooMainWindow.set_info_panel_text("Arg#/Destination: 0x" + Integer.toHexString(dest_arg) + "\n", style);
		VoodooMainWindow.set_info_panel_text("Cmd Arg (1-5): 0x" + Integer.toHexString(arg1) + ", 0x" + Integer.toHexString(arg2) +
								      ", 0x" + Integer.toHexString(arg3) + ", 0x" + Integer.toHexString(arg4) +
								      ", 0x" + Integer.toHexString(arg5) + "\n", style);
		VoodooMainWindow.set_info_panel_text("Return Value: 0x" + Integer.toHexString(return_value) + "\n", style);

		VoodooMainWindow.increment_info_panel_scrollbar();
	}

	/**
	********************************************************************************
	*	Converts a 24-bit integer into it's 3 character ascii equivalent. Used
	*	to output more descriptive DSP return values. The characters are checked
	*	to be between ascii 0x41 and 0x5A (A-Z).
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	private static String int_to_ascii(int value)
	{
		char ascii_value[] = new char[3];
		ascii_value[0] = (char)((value & 0xff0000) >> 16);
		ascii_value[1] = (char)((value & 0x00ff00) >> 8);
		ascii_value[2] = (char)(value & 0x0000ff);
		
		if (0x5A < (int)ascii_value[0] || (int)ascii_value[0] < 0x41)
			ascii_value[0] = (char)' ';
		if (0x5A < (int)ascii_value[1] || (int)ascii_value[1] < 0x41)
			ascii_value[1] = (char)' ';
		if (0x5A < (int)ascii_value[2] || (int)ascii_value[2] < 0x41)
			ascii_value[2] = (char)' ';			

		return (new String(ascii_value));
	}
}