/**
********************************************************************************
*	This class provides an exception if an error occurs during a read from
*	the reply buffer.
*
*	CVM - Model
*
*	@see Throwable
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
public class ReplyValueException extends Throwable
{
	private int offending_value;
	
	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@param offending_value The reply value that caused an error.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public ReplyValueException(int offending_value)
	{
		this.offending_value = offending_value;
	}
	
	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@param message The message that the exception should display.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public ReplyValueException(String message)
	{
		super(message);
	}
	
	/**
	********************************************************************************
	*	This method returns the reply value that caused an error.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public int get_value()
	{
		return offending_value;
	}
}
