/**
********************************************************************************
*	This class provides a file filter for loading setup files. This class
*	allows files containing a ".setup" extension to be the only ones
*	displayed in a file dialog box.
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import javax.swing.filechooser.FileFilter;
import java.io.*;

public class SetupFileFilter extends FileFilter
{
	private String filename;
	private String extension;
	private int period_index;
	private boolean accepted;
	
	/**
	********************************************************************************
	*	This method determines if any of the files in the current directory
	*	have the ".setup" extension. If they do, display there names in the file
	*	dialog box. Otherwise, toss them aside.
	*
	*	@see FileFilter
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public boolean accept(File file)
	{
		// Check if this is a directory and accept if it is.
		if (file.isDirectory())
			return true;
			
		// Retrieve the file name.
		filename = file.getName();
		
		// Get the location of the '.' in the filename.
		period_index = filename.lastIndexOf('.');
		
		// State of acceptence.
		accepted = false;
		
		if ((period_index > 0) && (period_index < (filename.length() - 1))) {
			extension = filename.substring(period_index + 1).toLowerCase();
			
			// Check for ".setup" files.
			if (extension.equals("setup"))
				accepted = true;
		}
		
		return accepted;
	}
	
	/**
	********************************************************************************
	*	This method returns the String that will be displayed in the file
	*	description section of the file dialog box.
	*
	*	@see FileFilter
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public String getDescription()
	{
		return "*.setup";
	}
}