/**
********************************************************************************
*	A class to display a splash screen during program startup. The
*	splash screen contains a background image which displays the
*	application name. The splash screen also displays the current
*	status of the application (i.e. what startup task the application
*	is currently performing).
*
*	CVM - View
*
*	@see JWindow
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.net.*;

public class SplashScreen extends JWindow implements Runnable
{
	private Container content_pane;
	private JLabel image_label;
	private Icon image_icon;
	private boolean done;
	private Thread thread;

	/**
	********************************************************************************
	*	Constructor. All the components such as the background image,
	*	the status label and the splash screen itself are created here.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public SplashScreen()
	{
		done = false;
		thread = new Thread(this, "Splash Screen");
		thread.start();
	}

	/**
	********************************************************************************
	*	This method overides the one from the <I>Runnable</I> interface.
	*	The splash screen is created, displayed, and undisplayed here.
	*
	*	@see Runnable
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void run()
	{
		// Center the window on the screen.
      		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      		setLocation(d.width/4, d.height/4);

		// Get a handle of the window's content pane.
		content_pane = this.getContentPane();
		
		// Set the window background & foreground colors.
		content_pane.setBackground(Color.black);
		content_pane.setForeground(Color.green);
	
		// Create the image icon.
		Image image = Toolkit.getDefaultToolkit().getImage(VoodooConstants.PIC_DIR + "VOODOO2.gif");
		image_icon = new ImageIcon(image);

		// Set the window's size.
		setSize(image.getWidth(null), image.getHeight(null)+15);
	
		// Create the image and status labels.
		image_label = new JLabel("", image_icon, JLabel.CENTER);
		JLabel status_label = new JLabel("  Initializing ... Please Wait");

		// Adjust the status label's properties.
		status_label.setFont(new Font("Courier", Font.BOLD, 14));
		status_label.setForeground(Color.white);

		// Add the image and status labels to the window.
		content_pane.add(image_label, BorderLayout.CENTER);
		content_pane.add(status_label, BorderLayout.SOUTH);
		
		// Make the splash screen visible.
		setVisible(true);
		
		// Wait for the parent application to finish with the splash
		// screen. Display any messages while waiting.
		while (!done) {
			try {
				Thread.sleep(1);
			}
			catch (InterruptedException ie) {}
		}
		
		// Make the splash screen invisible.
		setVisible(false);
	}

	/**
	********************************************************************************
	*	Closes the splash screen.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/	
	public void close_splash_screen()
	{
		done = true;
	}
}