/**
********************************************************************************
*	The main class of the Voodoo application.
*
*	CVM - Model
*
*	@see SplashScreen
*	@see VoodooMainWindow
*	@see VoodooSetup
*	@see MemorySetup
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import javax.swing.*;

public class Voodoo
{
	private VoodooMainWindow main_window = null;
	private SplashScreen splash_screen;

	/**
	********************************************************************************
	*	All startup procedures are performed here. Startup includes
	*	displaying the splash screen, setting up communications with
	*	the device driver, setting the kernel reply buffer, creating
	*	the image buffers and initializing and displaying the main
	*	application window.
	*
	*	@param args A list of command line arguments.
	*			
	*		    <br><pre>
	*		    args[0] = The path to the /Voodoo directory.
	*		    </pre>
	*
	*	@version 1.1
	*	@author Scott Streit
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	02/08/2000	sds	Removed reply buffer ready bit set.
	*	08/09/2000	sds	Removed path info for help files.
	********************************************************************************
	*/
	public static void main(String[] args)
	{
		Voodoo voodoo = new Voodoo();

		// Set the path information.
		try {
			VoodooConstants.PIC_DIR = args[0] + "/pics/";
		}
		catch(ArrayIndexOutOfBoundsException aioobe) {
			System.out.println("\nInstallation Path Information Missing\n");
			System.exit(0);
		}
		
     		// Initialize all non-window items.
		VoodooSetup setup = new VoodooSetup();

     		// Start the splash screen.
     		voodoo.splash_screen = new SplashScreen();
     		
		// Open the device driver connection.
		try {
			setup.initialize_device_driver();
    		}
    		catch (Exception e) {
			JOptionPane.showMessageDialog(voodoo.splash_screen, e.getMessage(), "Device Driver Error", JOptionPane.ERROR_MESSAGE);
       			System.exit(1);
    		}
		
   		// Initialize the reply buffer.
   		//ReplyBuffer.clear_reply();

   		// Create the image memory buffer.
   		MemorySetup mem_setup = new MemorySetup();
   		mem_setup.create_memory(VoodooConstants.IMAGE_BUFFER_SIZE);

		// Create the main voodoo windows.
		voodoo.main_window = new VoodooMainWindow("Voodoo");
		
		// Remove the splash screen and throw it away.
   		voodoo.splash_screen.close_splash_screen();
   		voodoo.splash_screen.dispose();
	}
}
