/**
********************************************************************************
*	This class defines global command and reply constants. It is declared
*	abstract since it contains only static constants and should never be
*	instantiated.
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
public abstract class VoodooConstants
{
	// Define PCI vector commands.
	public static final int  READ_CONTROLLER_STATUS	  = 	0x8079;
	public static final int  WRITE_CONTROLLER_STATUS  = 	0x807B;
	public static final int  RESET_CONTROLLER	  = 	0x807D;
	public static final int  LOAD_APPLICATION	  = 	0x807F;
	public static final int  PCI_PC_RESET             =	0x8081;
	public static final int  READ_PCI_STATUS          =	0x8083;
	public static final int  TEST_DATA_LINK           =	0x8085;
	public static final int  READ_MEMORY              =	0x8087;
	public static final int  WRITE_MEMORY             =	0x8089;
	public static final int  READ_CONTROLLER_CONFIG   =	0x808B;
	public static final int  POWER_ON		  =	0x808D;
	public static final int  POWER_OFF		  =	0x808F;
	public static final int  SET_BIAS_VOLTAGES	  =	0x8091;
	public static final int  CLEAR_ARRAY 		  =	0x8093;
	public static final int  STOP_IDLE_MODE		  =	0x8095;
	public static final int  RESUME_IDLE_MODE     	  =	0x8097;
	public static final int  READ_EXPOSURE_TIME	  =	0x8099;
	public static final int  START_EXPOSURE 	  =	0x809B;
	public static final int  READ_IMAGE     	  =	0x809D;
	public static final int  ABORT_READOUT            =	0x809F;
	public static final int  ABORT_EXPOSURE 	  =	0x80A1;
	public static final int  PAUSE_EXPOSURE		  =	0x80A3;
	public static final int  RESUME_EXPOSURE	  =	0x80A5;
	public static final int  RESERVED_2	     	  =	0x80A7;
	public static final int  OPEN_SHUTTER   	  =	0x80A9;
	public static final int  CLOSE_SHUTTER		  =	0x80AB;
	public static final int  SET_ARRAY_TEMPERATURE    =	0x80AD;
	public static final int  READ_ARRAY_TEMPERATURE   =	0x80AF;
	public static final int  PCI_DOWNLOAD		  =	0x802F;

	// Define replies.
	public static final int  REPLY_BUFFER_EMPTY	  =	-1;
	public static final int  TOUT			  =	0x544F5554;
	public static final int  DON                      =	0x00444F4E;
	public static final int  ERR			  =	0x00455252;
	public static final int  SYR                      =	0x00535952;
	public static final int  NULL 			  =	0x4E554C4C;		
	
	// Define controller/manual commands.
	public static final int  SRA                      =	0x00535241;
	public static final int  TDL                      =	0x0054444C;
	public static final int  RDM                      =	0x0052444D;
	public static final int  WRM                      =	0x0057524D;
	public static final int  WSI			  =	0x00575349;
	public static final int  SEX			  =	0x00534558;
	public static final int  PON			  =	0x00504F4E;
	public static final int  RDI			  =	0x00524449;
	public static final int  SOS			  =	0x00534F53;
	public static final int  MPP			  =	0x004D5050;
	public static final int  DCA			  =	0x00444341;
	public static final int  SNC			  =	0x00534E43;
	public static final int  VID			  =	0x00524944;
	public static final int  SBN			  =	0x0053424E;
	public static final int  SGN			  = 	0x0053474E;
	public static final int  RST                      =	0x00525354;

	// Define driver ioctl commands.
	public static final int  ASTROPCI_GET_REPLY	  =	0x3;
	public static final int  ASTROPCI_GET_HCTR	  =	0x5;
	public static final int  ASTROPCI_GET_HSTR        =     0x6;
	public static final int  ASTROPCI_GET_PROGRESS	  =	0x7;
	public static final int  ASTROPCI_GET_CONFIG_INFO =	0x314;
	public static final int  ASTROPCI_SET_CMDR  	  =	0x100;
	public static final int  ASTROPCI_SET_ARG1 	  =	0x105;
	public static final int  ASTROPCI_SET_ARG2 	  =	0x106;
	public static final int  ASTROPCI_SET_ARG3 	  =	0x107;
	public static final int  ASTROPCI_SET_ARG4 	  =	0x108;
	public static final int  ASTROPCI_SET_ARG5 	  =	0x109;
	public static final int  ASTROPCI_SET_ARGS 	  =	0x110;
	public static final int  ASTROPCI_SET_DESTINATION =	0x111;
	public static final int  ASTROPCI_SET_HCTR 	  =	0x115;
	public static final int  ASTROPCI_SET_HCVR	  =	0x117;
	public static final int  ASTROPCI_SET_EXPTIME	  =	0x119;
	public static final int  ASTROPCI_SET_NCOLS	  =	0x120;
	public static final int  ASTROPCI_SET_NROWS	  =	0x121;
	public static final int  ASTROPCI_SET_IMAGE_BUFFERS  =	0x122;
	public static final int  ASTROPCI_SET_UTIL_OPTIONS   =	0x123;
	public static final int  ASTROPCI_CLEAR_REPLY	     =	0x301;
	public static final int  ASTROPCI_ABORT_READ	     =	0x302;
	public static final int  ASTROPCI_FLUSH_REPLY_BUFFER =	0x124;
	public static final int  ASTROPCI_SET_BINNING_COLS   =	0x125;
	public static final int  ASTROPCI_SET_BINNING_ROWS   =	0x126;
		
	// Define board id's.
	public static final int  PCI_ID 	   	  =	1;
	public static final int  TIM_ID 	   	  =	2;
	public static final int  UTIL_ID 	   	  =	3;
	
	// Define argument numbers for commands that use board destination.
	public static final int  TDL_ARG_NUM		  =	0x10000;	// Number of "Argument" registers used by TDL
	public static final int  RDM_ARG_NUM		  =	0x20000;	// Number of "Argument" registers used by RDM
	public static final int  WRM_ARG_NUM		  =	0x30000;	// Number of "Argument" registers used by WRM
	public static final int  TIM_APP_ARG_NUM	  =	0x10000;
	public static final int  UTIL_APP_ARG_NUM	  =	0x10000;
	
	// General constants.
	public static final int  _OPEN_SHUTTER_           =	(1 << 11);
	public static final int  _CLOSE_SHUTTER_          =	~(1 << 11);
	public static final int  IMAGE_BUFFER_SIZE	  =	9680000;	// Default image buffer size (bytes)
	public static final int  AUTO                     =	800;
	public static final int  MANUAL		   	  =	801;
	public static final int  SUCCESS		  =	0;
	public static final int  FAILURE		  =	1;
	public static final int  YES			  =	1;
	public static final int  NO			  =	0;		
	public static final boolean  DEBUG_ON		  =	true;
	public static final boolean  DEBUG_OFF		  =	false;
	public static final boolean  ON			  =	true;
	public static final boolean  OFF		  =	false;

	// Controller configuration bit masks.
	public static final int  VIDEO_PROCESSOR_MASK	  =	0x000007;
	public static final int  TIMING_BOARD_MASK	  =	0x000018;
	public static final int  UTILITY_BOARD_MASK	  =	0x000060;
	public static final int  SHUTTER_MASK		  =	0x000080;
	public static final int  TEMPERATURE_READOUT_MASK =	0x000300;
	public static final int  SUBARRAY_READOUT_MASK	  =	0x000400;
	public static final int  BINNING_MASK		  =	0x000800;
	public static final int  READOUT_MASK		  =	0x003000;
	public static final int  MPP_MASK		  =	0x004000;

	// Controller configuration bit definitions.
	public static final int  DEFAULT_CONFIG_WORD	  =	0x000080;
	public static final int  CCD_REV3B		  =	0x000000;
	public static final int  CCD_GENI		  =	0x000001;
	public static final int  CCD			  =	0x000000;
	public static final int  IR_REV4C		  =	0x000002;
	public static final int  IR_COADDER		  =	0x000003;
	public static final int  IR			  =	0x000001;
	public static final int  TIM_REV4B		  =	0x000000;
	public static final int  TIM_GENI		  =	0x000008;
	public static final int  NO_UTIL_BOARD		  =	0x000000;
	public static final int  UTILITY_REV3		  =	0x000020;
	public static final int  SHUTTER		  =	0x000080;
	public static final int  NO_TEMPERTURE_CONTROL	  =	0x000000;
	public static final int  NONLINEAR_TEMP_CONV	  =	0x000100;
	public static final int  LINEAR_TEMP_CONV	  =	0x000200;
	public static final int  SUBARRAY		  =	0x000400;
	public static final int  BINNING		  =	0x000800;
        public static final int  NO_SPLIT                 =     0x000000;
        public static final int  SERIAL_SPLIT             =     0x001000;
        public static final int  PARALLEL                 =     0x002000;
        public static final int  BOTH_READOUTS            =     0x003000;
	public static final int  MPP_CAPABLE		  =	0x004000;

        // Select output source ('SOS') and memory type constants.
        public static final int  A                        =	0x5F5F41;       //  Ascii __A amp A.
        public static final int  B                        = 	0x5F5F42;       //  Ascii __B amp B.
        public static final int  C                        =	0x5F5F43;       //  Ascii __C amp C.
        public static final int  L                        = 	0x5F5F4C;       //  Ascii __L left amp.
        public static final int  LR                       =	0x5F4C52;       //  Ascii _LR right two amps.
        public static final int  AB                       =	0x5F4142;       //  Ascii _AB top two amps A & B.
        public static final int  CD                       =	0x5F4344;       //  Ascii _CD bottom two amps C & D.
        public static final int  ALL                      =	0x414C4C;       //  Ascii ALL four amps (quad).
	public static final int  D                        =	0x5F5F44;	//  Ascii __D
	public static final int  R                        =	0x5F5F52;	//  Ascii __R
	public static final int  P                        =	0x5F5F50;	//  Ascii __P
	public static final int  X                        =	0x5F5F58;	//  Ascii __X
	public static final int  Y                        =	0x5F5F59;	//  Ascii __Y

	// Define CCD setup constants.
	public static final int  HARDWARE_DATA_MAX        =	1000000;

	// Define default temperature constants.
	public static final double NONLINEAR_TEMP_COEFF0  = 	287.756797;
	public static final double NONLINEAR_TEMP_COEFF1  = 	-194.144823;
	public static final double NONLINEAR_TEMP_COEFF2  =	-3.837903;
	public static final double NONLINEAR_TEMP_COEFF3  = 	-1.318325;
	public static final double NONLINEAR_TEMP_COEFF4  =	-0.109120;
	public static final double NONLINEAR_TEMP_COEFF5  =	-0.393265;
	public static final double NONLINEAR_TEMP_COEFF6  =	0.146911;
	public static final double NONLINEAR_TEMP_COEFF7  = 	-0.111192;
	public static final double NONLINEAR_TEMP_COEFF8  = 	0.028877;
	public static final double NONLINEAR_TEMP_COEFF9  = 	-0.029286;
	public static final double NONLINEAR_TEMP_COEFF10 = 	0.015619;
	public static final double NONLINEAR_ADU_PER_VOLT = 	1366.98;
	public static final double NONLINEAR_ADU_OFFSET   = 	2045;
	public static final double NONLINEAR_VU		  = 	0.999614;
	public static final double NONLINEAR_VL		  = 	0.079767;
	public static final double TOLERANCE		  = 	0.5;
	public static final double MAX_TOLERANCE_TRIALS	  = 	30;

	// Define gain and speed constants.
	public static final int  ONE		  	  =	1;
	public static final int  TWO		  	  =	2;
	public static final int  FIVE		  	  =	5;
	public static final int  TEN		  	  =	10;
	public static final int  SLOW		  	  =	0;
	public static final int  FAST		  	  =	1;
	
	// Non-constant system resources.
	public static String PIC_DIR		  	  =	"../pics/";

	// Define window constants.
	public static int  WINDOW_PADDING	  	  =	8;		// Pixels
	
	public VoodooConstants() {}
}
