/**
********************************************************************************
*	The class is responsible for responding to events from the exposure
*	control section of the main application window.
*
*	CVM - Controller
*
*	@see ItemListener
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;

public class VoodooGlobalSetupListener implements ItemListener, ActionListener, ChangeListener
{
	protected VoodooGlobalSetupDialog parent;
	private VoodooGlobalSetupSave global_save;
	private JFileChooser file_chooser = null;
	private	int selectedIndex;
	private String source;
	
	/**
	*****************************************************************************
	*	Constructor.
	*
	*	@param parent The window for which this controller listens.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public VoodooGlobalSetupListener(VoodooGlobalSetupDialog parent)
	{
		this.parent = parent;
		global_save = new VoodooGlobalSetupSave(this.parent);
		selectedIndex = -1;
	}

	/**
	*****************************************************************************
	*	This method handles item selection events from the Global Setup
	*	section of the main application window.
	*
	*	@param ie The selection event.
	*
	*	@see ItemListener
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void itemStateChanged(ItemEvent ie)
	{
		JComboBox active_value = (JComboBox) ie.getSource();

		// Set the source.
		source = (String)active_value.getSelectedItem();
		
		if (source.equals("Controller Setup Window")) {
	 		parent.width_txtfld.setText(Integer.toString((int)parent.parent.setup_dialog.getSize().getWidth()));
	 		parent.height_txtfld.setText(Integer.toString((int)parent.parent.setup_dialog.getSize().getHeight()));
		}
		else if (source.equals("Voodoo Main Window")) {
	 		parent.width_txtfld.setText(Integer.toString((int)parent.parent.getSize().getWidth()));
	 		parent.height_txtfld.setText(Integer.toString((int)parent.parent.getSize().getHeight()));
	 	}
		else if (source.equals("DSP Commands Window")) {
	 		parent.width_txtfld.setText(Integer.toString((int)parent.parent.dsp_dialog.getSize().getWidth()));
	 		parent.height_txtfld.setText(Integer.toString((int)parent.parent.dsp_dialog.getSize().getHeight()));
		}
		else if (source.equals("FITS Window")) {
	 		parent.width_txtfld.setText(Integer.toString((int)parent.parent.fits_dialog.getSize().getWidth()));
	 		parent.height_txtfld.setText(Integer.toString((int)parent.parent.fits_dialog.getSize().getHeight()));
		}
		else if (source.equals("Image Buffer Window")) {
	 		parent.width_txtfld.setText(Integer.toString((int)parent.parent.image_buffers_dialog.getSize().getWidth()));
	 		parent.height_txtfld.setText(Integer.toString((int)parent.parent.image_buffers_dialog.getSize().getHeight()));
		}
		else if (source.equals("Global Options Window")) {
	 		parent.width_txtfld.setText(Integer.toString((int)parent.parent.voodoo_global_setup_dialog.getSize().getWidth()));
	 		parent.height_txtfld.setText(Integer.toString((int)parent.parent.voodoo_global_setup_dialog.getSize().getHeight()));
		}
		else if (source.equals("Hardware Test Window")) {
	 		parent.width_txtfld.setText(Integer.toString((int)parent.parent.setup_dialog.hardware_test_dialog.getSize().getWidth()));
	 		parent.height_txtfld.setText(Integer.toString((int)parent.parent.setup_dialog.hardware_test_dialog.getSize().getHeight()));
		}
		else if (source.equals("Controller Config Window")) {
	 		parent.width_txtfld.setText(Integer.toString((int)parent.parent.setup_dialog.controller_config_dialog.getSize().getWidth()));
	 		parent.height_txtfld.setText(Integer.toString((int)parent.parent.setup_dialog.controller_config_dialog.getSize().getHeight()));
		}
	}

	/**
	********************************************************************************
	*	This method handles all non-keyboard events.
	*
	*	@param e The event.
	*
	*	@see ActionListener
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void actionPerformed(ActionEvent e)
	{
		AbstractButton active_button = (AbstractButton) e.getSource();
		File file;
		int width;
		int height;

		// Set the specified window size.
		if (active_button.getText().equals("Apply Window Size")) {
			if(parent.width_txtfld.getText().equals("") || parent.height_txtfld.getText().equals(""))
				JOptionPane.showMessageDialog(parent, "Both window width and height must be entered.", "Error", JOptionPane.ERROR_MESSAGE);
			else {
				width = (int)Integer.parseInt(parent.width_txtfld.getText());
				height = (int)Integer.parseInt(parent.height_txtfld.getText());
						
				if (source.equals("Controller Setup Window")) 
					parent.parent.setup_dialog.setSize(width, height);
				else if (source.equals("Voodoo Main Window")) 
					parent.parent.setSize(width, height);
				else if (source.equals("DSP Commands Window")) 
					parent.parent.dsp_dialog.setSize(width, height);
				else if (source.equals("FITS Window")) 
					parent.parent.fits_dialog.setSize(width, height);
				else if (source.equals("Image Buffer Window")) 
					parent.parent.image_buffers_dialog.setSize(width, height);
				else if (source.equals("Global Options Window")) 
					parent.parent.voodoo_global_setup_dialog.setSize(width, height);
				else if (source.equals("Hardware Test Window"))
	 				parent.parent.setup_dialog.hardware_test_dialog.setSize(width, height);
				else if (source.equals("Controller Config Window"))
	 				parent.parent.setup_dialog.controller_config_dialog.setSize(width, height);
			}
		}
		
		// Close the window.
		else if (active_button.getText().equals("Close")) {
			parent.parent.voodoo_global_setup_dialog.setVisible(false);
			parent.parent.voodoo_global_setup_dialog.number_of_global_setup_windows = 0;
		}

		// Save the global settings.
		else if (active_button.getText().equals("Save Settings")) {
			try {
				global_save.write_to_file();

				// Inform user that setup is complete.
				PrintMessage.message("VoodooGlobalSetupListener", 0, "Global setup saved to .VoodooChild.curse\n", 0, true);
			}
			catch (IOException ioe) {
				JOptionPane.showMessageDialog(parent, "Settings not saved, error writing file.", "Error", JOptionPane.ERROR_MESSAGE);
			}
		}
		
		// Browse for a Controller Setup file.
		else if (active_button.getActionCommand().equals("Controller Setup Browse")) {
			// Open a file chooser that points to the current dir.
			file_chooser = new JFileChooser(".");
							
			// Add a file filter.
			file_chooser.addChoosableFileFilter(new SetupFileFilter());
			
			// Assign the file filter and discard all others.
			file_chooser.setFileFilter(new SetupFileFilter());
			
			// Set the Open dialog box size.
			file_chooser.setPreferredSize(new Dimension(500, 250));
			
			// Show the Open dialog box (returns the option selected)
			int selected = file_chooser.showOpenDialog(parent.content_pane);

			// If the Open button is pressed.
			if (selected == JFileChooser.APPROVE_OPTION) {
				// Get the selected file.
				file = file_chooser.getSelectedFile();
		
				// Display the file name in the frame.
				parent.controller_file_txtfld.setText(file.getPath());
				return;
			}
			// If the Cancel button is pressed.
			else if (selected == file_chooser.CANCEL_OPTION) {
				return;
			}
		}
	}

	/**
	*****************************************************************************
	*	Controls the switching of the "tabbed panes". This method
	*	overides the same method in the ChangeListener interface.
	*
	*	@see ChangeListener
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void stateChanged(ChangeEvent ce)
   	{
      		JTabbedPane tp = (JTabbedPane)ce.getSource();
       
      		// Check if a tab has been selected for the first time or if a new tab
      		// has been selected and set the tab to "false".
      		if ((selectedIndex == -1) || (selectedIndex != tp.getSelectedIndex())) {
      			tp.setEnabledAt(tp.getSelectedIndex(), false);
         	
         		// If this is not the first tab selection, then set the previously
         		// selected tab to "true".
         		if (selectedIndex != -1)
         			tp.setEnabledAt(selectedIndex, true);
      		}
      		// Store the index of the newly selected tab.
      		selectedIndex = tp.getSelectedIndex();
   	}
}
