/**
********************************************************************************
*	A class to load or save the current Voodoo settings. Window sizes, all
*	exposure settings from the main window and the Controller Setup Window
*	setup filename is saved here. The settings are saved in a text file
*	called ".VoodooChild.curse".
*
*	CVM - Model
*
*	@see SetupDialog
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.util.*;

public class VoodooGlobalSetupSave
{
	private VoodooGlobalSetupDialog parent;
	private SetupSave setup_save;
	private final String filename = ".VoodooChild.curse";

	/**
	*****************************************************************************
	*	Default Constructor.
	*****************************************************************************
	*/
	public VoodooGlobalSetupSave(VoodooGlobalSetupDialog parent)
	{
		this.parent = parent;
		setup_save = new SetupSave();
	}

	/**
	*****************************************************************************
	*	Saves the current component settings in the CCD Setup Window to the
	*	specified text file. The file is specified via a popup JFileChooser
	*	dialog.
	*
	*	@param parent_component The container that holds the CCD Setup Window
	*		                components (i.e. content pane, panel, etc).
	*	@param setup_dialog The CCD Setup window.
	*
	*	@throws IOException
	*	@see SetupDialog
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void write_to_file() throws IOException
	{
		RandomAccessFile outFile;
		File file = new File(filename);

		outFile = new RandomAccessFile(file, "rw");
			
		// Set the file length to 1 byte. This is to ensure the new
		// data is properly written.
		outFile.setLength(1);
				
		// Save the Controller Setup window dimensions.
		outFile.writeBytes(parent.parent.setup_dialog.getClass().getName() + " " + (int)parent.parent.setup_dialog.getSize().getWidth() + "\n");
		outFile.writeBytes(parent.parent.setup_dialog.getClass().getName() + " " + (int)parent.parent.setup_dialog.getSize().getHeight() + "\n");

		// Save the Main window dimensions.
		outFile.writeBytes(parent.parent.getClass().getName() + " " + (int)parent.parent.getSize().getWidth() + "\n");
		outFile.writeBytes(parent.parent.getClass().getName() + " " + (int)parent.parent.getSize().getHeight() + "\n");

		// Save the DSP Command window dimensions.
		outFile.writeBytes(parent.parent.dsp_dialog.getClass().getName() + " " + (int)parent.parent.dsp_dialog.getSize().getWidth() + "\n");
		outFile.writeBytes(parent.parent.dsp_dialog.getClass().getName() + " " + (int)parent.parent.dsp_dialog.getSize().getHeight() + "\n");

		// Save the FITS window dimensions.
		outFile.writeBytes(parent.parent.fits_dialog.getClass().getName() + " " + (int)parent.parent.fits_dialog.getSize().getWidth() + "\n");
		outFile.writeBytes(parent.parent.fits_dialog.getClass().getName() + " " + (int)parent.parent.fits_dialog.getSize().getHeight() + "\n");

		// Save the Image Buffers window dimensions.
		outFile.writeBytes(parent.parent.image_buffers_dialog.getClass().getName() + " " + (int)parent.parent.image_buffers_dialog.getSize().getWidth() + "\n");
		outFile.writeBytes(parent.parent.image_buffers_dialog.getClass().getName() + " " + (int)parent.parent.image_buffers_dialog.getSize().getHeight() + "\n");

		// Save the Global Setup window dimensions.
		outFile.writeBytes(parent.parent.voodoo_global_setup_dialog.getClass().getName() + " " + (int)parent.parent.voodoo_global_setup_dialog.getSize().getWidth() + "\n");
		outFile.writeBytes(parent.parent.voodoo_global_setup_dialog.getClass().getName() + " " + (int)parent.parent.voodoo_global_setup_dialog.getSize().getHeight() + "\n");

		// Save the Hardware Test Options window dimensions.
		outFile.writeBytes(parent.parent.setup_dialog.hardware_test_dialog.getClass().getName() + " " + (int)parent.parent.setup_dialog.hardware_test_dialog.getSize().getWidth() + "\n");
		outFile.writeBytes(parent.parent.setup_dialog.hardware_test_dialog.getClass().getName() + " " + (int)parent.parent.setup_dialog.hardware_test_dialog.getSize().getHeight() + "\n");

		// Save the Controller Configuration window dimensions.
		outFile.writeBytes(parent.parent.setup_dialog.controller_config_dialog.getClass().getName() + " " + (int)parent.parent.setup_dialog.controller_config_dialog.getSize().getWidth() + "\n");
		outFile.writeBytes(parent.parent.setup_dialog.controller_config_dialog.getClass().getName() + " " + (int)parent.parent.setup_dialog.controller_config_dialog.getSize().getHeight() + "\n");

		outFile.writeBytes("****************** Controller Setup *************\n");

		// Save the Controller Setup file name.
		if (parent.controller_file_txtfld.getText().equals(""))
			outFile.writeBytes("\n");
		else
			outFile.writeBytes(parent.controller_file_txtfld.getText() + "\n");

		outFile.writeBytes("*************** Main Window Image Options **********************\n");

		// Save the Main Window Image Option checkboxes.
		outFile.writeBytes(parent.parent.display_image_chkbox.getText() + "\t" + String.valueOf(parent.parent.display_image_chkbox.isSelected()) + "\n");
		outFile.writeBytes(parent.parent.delay_chkbox.getText() + "\t" + String.valueOf(parent.parent.delay_chkbox.isSelected()) + "\n");
		outFile.writeBytes(parent.parent.multi_exp_chkbox.getText() + "\t" + String.valueOf(parent.parent.multi_exp_chkbox.isSelected()) + "\n");
		outFile.writeBytes(parent.parent.save_disk_chkbox.getText() + "\t" + String.valueOf(parent.parent.save_disk_chkbox.isSelected()) + "\n");
		outFile.writeBytes(parent.parent.autoincr_chkbox.getText() + "\t" + String.valueOf(parent.parent.autoincr_chkbox.isSelected()) + "\n");

		// Save the Main Window Image Option textfields.
		outFile.writeBytes("Delay txtfld" + "\t" + parent.parent.delay_txtfld.getText() + "\n");
		outFile.writeBytes("Multiple Exp txtfld" + "\t" + parent.parent.mult_exp_txtfld.getText() + "\n");
		outFile.writeBytes("Image txtfld" + "\t" + parent.parent.image_filename_txtfld.getText() + "\n");

		// Save the deinterlace combo box info.
		outFile.writeBytes("Deinterlace " + "\t" + String.valueOf(parent.parent.deinterlace_combo_box.getSelectedIndex()) + "\n");

		outFile.writeBytes("*************** Main Window Exposure Control **********************\n");

		// Save the Main Window Image Option checkboxes.
		outFile.writeBytes(parent.parent.open_shutter_chkbox.getText() + "\t" + String.valueOf(parent.parent.open_shutter_chkbox.isSelected()) + "\n");
		outFile.writeBytes(parent.parent.beep_chkbox.getText() + "\t" + String.valueOf(parent.parent.beep_chkbox.isSelected()) + "\n");

		// Save the Main Window Image Option textfields.
		outFile.writeBytes("Exposure Time txtfld" + "\t" + parent.parent.exposure_time_txtfld.getText() + "\n");

		outFile.close();
	}

	/**
	*****************************************************************************
	*	Loads the component settings, previously stored in the specified
	*	text file, into the CCD Setup Window. The file is specified via a
	*	popup JFileChooser dialog. The window sizes are	padded with
	*	VoodooConstants.WINDOW_PADDING to ensure they display without being
	*	"squashed".
	*
	*	@param parent_component The container that holds the CCD Setup Window
	*		      components (i.e. content pane, panel, etc).
	*	@param setup_dialog The CCD Setup window.
	*
	*	@throws IOException
	*	@see SetupDialog
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void read_from_file() throws IOException
	{
		RandomAccessFile inFile;
		JFileChooser file_chooser;
		File file = new File(filename);
		boolean selected;
		String line;
		int width;
		int height;
	
		// Check that the global setup file exists. If not, exit.
		if (!file.exists())
			return;
			
		// Open the file and output.
		inFile = new RandomAccessFile(file, "r");

		// Read the Controller Setup window dimensions.
		line = inFile.readLine();
		width = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		line = inFile.readLine();
		height = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		parent.parent.setup_dialog.setSize(width, height);
			
		// Read the Main window dimensions.
		line = inFile.readLine();
		width = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		line = inFile.readLine();
		height = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		parent.parent.setSize(width, height);

		// Read the DSP Command window dimensions.
		line = inFile.readLine();
		width = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		line = inFile.readLine();
		height = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		parent.parent.dsp_dialog.setSize(width, height);

		// Read the FITS window dimensions.
		line = inFile.readLine();
		width = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		line = inFile.readLine();
		height = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		parent.parent.fits_dialog.setSize(width, height);

		// Read the Image Buffers window dimensions.
		line = inFile.readLine();
		width = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		line = inFile.readLine();
		height = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		parent.parent.image_buffers_dialog.setSize(width, height);

		// Read the Global Setup window dimensions.
		line = inFile.readLine();
		width = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		line = inFile.readLine();
		height = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		parent.parent.voodoo_global_setup_dialog.setSize(width, height);

		// Read the Hardware Test Options window dimensions.
		line = inFile.readLine();
		width = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		line = inFile.readLine();
		height = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		parent.parent.setup_dialog.hardware_test_dialog.setSize(width, height);

		// Read the Controller Configuration window dimensions.
		line = inFile.readLine();
		width = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		line = inFile.readLine();
		height = Integer.parseInt(line.substring(line.indexOf(" ")+1)) + VoodooConstants.WINDOW_PADDING;
		parent.parent.setup_dialog.controller_config_dialog.setSize(width, height);

		// Read stars.
		line = inFile.readLine();

		// Read the Controller Setup file name.
		line = inFile.readLine();
		if (line.equals("")) ; // Do nothing
		else {
			setup_save.read_file(new File(line), parent.parent.setup_dialog, parent.parent.setup_dialog.hardware_test_dialog);
			parent.parent.setup_dialog.clickApply();

			// Wait for the controller setup to complete before attempting to initialize the
			// controller configuration window. A NullPointerException will occur if the
			// timing file/app has not been applied first.
			while (parent.parent.setup_dialog.setup_listener.setup_cmd.alive()) {
				try {
					Thread.sleep(5);
				}
				catch (InterruptedException ie) {}
			}
			// Do the controller configuration settings.
			setup_save.read_file(new File(line), parent.parent.setup_dialog.controller_config_dialog);
			parent.parent.setup_dialog.controller_config_dialog.clickApplyAll();
		}
			
		// Read stars.
		line = inFile.readLine();

		// Read the Main Window Image Option checkboxes.
		line = inFile.readLine();
		parent.parent.display_image_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());
		line = inFile.readLine();
		parent.parent.delay_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());
		line = inFile.readLine();
		parent.parent.multi_exp_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());
		line = inFile.readLine();
		parent.parent.save_disk_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());
		line = inFile.readLine();
		parent.parent.autoincr_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());

		// Read the Main Window Image Option textfields.
		line = inFile.readLine();
		parent.parent.delay_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));
		line = inFile.readLine();
		parent.parent.mult_exp_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));
		line = inFile.readLine();
		parent.parent.image_filename_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

		// Set the deinterlace combo box info.
		line = inFile.readLine();
		parent.parent.deinterlace_combo_box.setSelectedIndex(new Integer(line.substring(line.lastIndexOf("\t")+1)).intValue());

		// Read stars.
		line = inFile.readLine();

		// Read the Main Window Image Option checkboxes.
		line = inFile.readLine();
		parent.parent.open_shutter_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());
		line = inFile.readLine();
		parent.parent.beep_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());

		// Read the Main Window Image Option textfields.
		line = inFile.readLine();
		parent.parent.exposure_time_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

		// Read stars.
		line = inFile.readLine();

		inFile.close();
	}
}
