/**
********************************************************************************
*	This class provides initialization for all non-window objects that need 
*	initialization. In particular, the device driver C library functions are
*	enabled here along with any global constants and variables that are needed.
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import javax.swing.*;
import java.io.*;

public class VoodooSetup
{
  	private static final String  PCI_DEV	=  "/dev/astropci0";
  	public static int pci_fd;
	
  	// C function prototypes.
  	private native int Copen(String pci_dev);
	private static native int Cclose(int fd);
	public static native int Cioctl(int fd, int req, int dat);
	public static native int Cread_image(int fd, int mem_fd, int read_size);
	public static native int Cread_reply(int fd, int dat);

  	// Load the library that contains the C functions.
  	static
  	{
    		System.loadLibrary("pcilibc");
  	}

	/**
	********************************************************************************
	*	Constructor.
	*
	*	@version 1.1
	*	@author Scott Streit
	********************************************************************************
	*/
  	public VoodooSetup()
  	{
    		pci_fd = 0;
  	}

	/**
	********************************************************************************
	*	Closes the connection to the device driver.
	*
	*	@version 1.1
	*	@author Scott Streit
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	07/11/2000	sds	Moved exception catch to outside the function.
	********************************************************************************
	*/
	public static void close_device_driver() throws Exception
	{
		Cclose(pci_fd);
		pci_fd = 0;
	}
	
	/**
	********************************************************************************
	*	Opens the connection to the device driver for PCI board access.
	*
	*	@version 1.1
	*	@author Scott Streit
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	07/11/2000	sds	Moved exception catch to outside the function.
	********************************************************************************
	*/
  	public void initialize_device_driver() throws Exception
  	{
		pci_fd = Copen(PCI_DEV);
  	}
}
